#include <iostream>
#include <string>
#include <list>

#include <AlibException.h>
#include <automaton/AutomatonParser.h>
#include <automaton/UnknownAutomaton.h>
#include <AutomatonFactory.h>
#include <sax/SaxInterface.h>

#include "NFARun.h"

using namespace alib;
using namespace automaton;
using namespace alphabet;
using namespace std;
using namespace sax;

/*
 * Usage:
 *  aacept.fsm "a" "b" "cc" < fsm.xml (accepts FSM.xml word <a><b><cc> ?)
 */

int main(int argc, char** argv)
{
    try
    {
        list<Token> tokens;
        string input(istreambuf_iterator<char>(cin), (istreambuf_iterator<char>()));
        SaxInterface::parseMemory(input, tokens);

        UnknownAutomaton unknownAutomaton = AutomatonParser::parse(tokens);
        FSM fsm = AutomatonFactory::buildFSM( unknownAutomaton );
        NFARun nfa( fsm );
        list<Symbol> word;

        for( int i = 1; i < argc ; i++ )
            word.push_back( Symbol ( argv[ i ] ) );

        if( nfa.simulate( word ) )
        {
            cout << "Automaton accepted word." << endl;
            return 0;
        }
        else
        {
            cout << "Automaton did not accept word" << endl;
            return 1;
        }
    }
    catch( AlibException & e )
    {
        cout << e.what() << endl;
        return 255;
    }

    return 0;
}
