#ifndef FSMDETERMINIZER_H_
#define FSMDETERMINIZER_H_

#include <string>
#include <set>
#include <map>

#include "automaton/State.h"
#include "automaton/FSM/FSM.h"
#include "automaton/FSM/TransitionFSM.h"

#include "../common/Utils.h"
#include "../Determinizer.h"
#include "FsmStructs.h"
#include "FsmUtils.h"

using namespace std;
using namespace automaton;
using namespace determinization;

namespace determinization {
namespace fsm {

/**
 * Class for running determinization algorithm on fsm.
 */
class FsmDeterminizer : public Determinizer
{

    private:

        /** Nondeterministic final-state maching */
        FSM* nfsm;

        /** Deterministinc final-state machine */
        FSM* dfsm;

        /** Map of states of deterministic fsm, where key is the name of state and value is data about this state */
        map<string, StateData> QPrimed;
        
        /**
         * Runs some initializiation stuff before determinization algorithm.
         */
        void initDeterminization();

        /**
         * Returns existing state from states map, if there is one with same name, or creates new one and adds it into
         *     states map and deterministic fsm.
         *
         * @param originalStates set of states from nondeterministic fsm which represents state of deterministic fsm
         * @return state of deterministic fsm
         */
        const State& getOrCreateState(const set<State>& originalStates);

        /**
         * Finds states from nondeterministic fsm to which at least one state from given set of states have transition
         *     with given input.
         * 
         * @param fromStates set of states from nondeterministic fsm
         * @param input symbol from input alphabet
         * @return set of states from nondeterministic fsm
         */
        set<State> findToStates(const set<State>& fromStates, const Symbol& input);
        
    public:

        /**
         * @param nfsm nondeterministic final-state machine given for determinization
         */
        FsmDeterminizer(FSM* nfsm);

        /**
         * Runs determinization algorithm on nondeterministic fsm given in constructor.
         *
         * @return deterministic final-state machine
         */
        Automaton* determinize();

};

}
}
#endif
