#include "FsmUtils.h"

namespace determinization {
namespace fsm {


string FsmUtils::buildStateName(const set<State>& states)
{
    string name = "[";
    for (const auto& state : states) {
        if (&state != &(*begin(states))) {
            name += ", ";
        }
        name += "'" + state.getName() + "'";
    }
    name += "]";
    return name;
}


const State& FsmUtils::getOrCreateState(const set<State>& originalStates, map<string, StateData>& states,
    Automaton& automaton)
{
    string stateName = buildStateName(originalStates);
    
    map<string, StateData>::iterator stateIter = states.find(stateName);
    if (stateIter != states.end()) {
        return stateIter->second.state;
    }
    
    State state(stateName);
    StateData stateData(state, originalStates);
    StateData& insertedData = states.insert(pair<string, StateData>(stateName, stateData)).first->second;
    automaton.addState(state);
    return insertedData.state;
}


}
}
