#ifndef FSMUTILS_H_
#define FSMUTILS_H_

#include <string>
#include <set>
#include <map>

#include "automaton/State.h"
#include "automaton/Automaton.h"
#include "FsmStructs.h"

using namespace std;
using namespace automaton;
using namespace determinization;

namespace determinization {
namespace fsm {

/**
 * Library class with some methods that can be used by fsm and idpda determinization.
 */
class FsmUtils
{       
    
    public:

        /**
         * Returns name of state of deterministic automaton which is created from given set of states
         *     from nondeterministic automaton.
         *
         * @param states set of states from nondeterministinc fsm
         * @return name of state
         */
        static string buildStateName(const set<State>& states);

        /**
         * Returns existing state from states map, if there is one with same name, or creates new one and adds it into
         *     states map and automaton.
         *
         * @param originalStates set of states from nondeterministic automaton which represents state
         *     of deterministic automton
         * @param states map of existing states, where key is the name of state and value is data about this state
         * @param automaton deterministic automaton that contains given map of states
         * @return state of deterministic automaton
         */
        static const State& getOrCreateState(const set<State>& originalStates, map<string, StateData>& states,
            Automaton& automaton);

};

}
}
#endif
