#include "IdpdaDeterminizer.h"

namespace determinization {
namespace idpda {


void IdpdaDeterminizer::initDeterminization()
{
    Utils::copyInputAlphabet(*this->nidpda, *this->didpda);
    Utils::copyStackAlphabet(*this->nidpda, *this->didpda);
    this->QPrimed.clear();
    this->findPopAndPushSymbols();
}


void IdpdaDeterminizer::findPopAndPushSymbols()
{
    for (const auto& symbol: this->nidpda->getInputAlphabet()) {
        for (const auto& transition: this->nidpda->getTransitions()) {
            if (transition.getInput() == symbol) {
                this->popSymbols[symbol] = transition.getPop();
                this->pushSymbols[symbol] = transition.getPush();
                break;
            }
        }
    }
}


const State& IdpdaDeterminizer::getOrCreateState(const set<State>& originalStates)
{
    return FsmUtils::getOrCreateState(originalStates, this->QPrimed, *this->didpda);
}


set<State> IdpdaDeterminizer::findToStates(const set<State>& fromStates, const Symbol& input)
{
    set<State> targetStates;
    for (const auto& state : fromStates) {
        for (const auto& transition : this->nidpda->getTransitions()) {
            if (transition.getFrom() == state && transition.getInput() == input) {
                targetStates.insert(transition.getTo());
            }
        }
    }
    return targetStates;
}


IdpdaDeterminizer::IdpdaDeterminizer(PDA* nidpda)
{
    this->nidpda = nidpda;
}


Automaton* IdpdaDeterminizer::determinize()
{
    this->didpda = new PDA();
    this->initDeterminization();
    
    // 1
    const set<State>& q0 = this->nidpda->getInitialStates();
    const State& q0Primed = this->getOrCreateState(q0);
    
    while (true) {
        // 2
        bool allStatesMarked = true;
        for (const auto& stateIter : this->QPrimed) {
            allStatesMarked &= stateIter.second.isMarked;
        }
        if (allStatesMarked) {
            break;
        }
        
        // 3
        StateData* qPrimed = NULL;
        for (auto& stateIter : this->QPrimed) {
            StateData& stateData = stateIter.second;
            if (!stateData.isMarked) {
                qPrimed = &stateData;
                break;
            }
        }
        
        // 3a
        for (const auto& input : this->nidpda->getInputAlphabet()) {
            list<Symbol> stackPop = this->popSymbols[input];
            list<Symbol> stackPush = this->pushSymbols[input];
            const set<State>& nidpdaToStates = this->findToStates(qPrimed->originalStates, input);
            if (nidpdaToStates.size() > 0) {
                const State& toState = this->getOrCreateState(nidpdaToStates);
                TransitionPDA transition(qPrimed->state, input, toState, stackPop, stackPush);
                this->didpda->addTransition(transition);
            }
        }
        
        // 3b
        qPrimed->isMarked = true;
    }
    
    // 4
    this->didpda->addInitialState(q0Primed);
    
    // 5
    for (const auto& qPrimedIter : this->QPrimed) {
        const StateData& qPrimed = qPrimedIter.second;
        if (Utils::containFinalState(qPrimed.originalStates, *this->nidpda)) {
            this->didpda->addFinalState(qPrimed.state);
        }
    }
    
    return this->didpda;
}


}
}
