/*
 * AutomatonDiff.h
 *
 *  Created on: Apr 1, 2013
 *      Author: honza
 */

#ifndef AUTOMATON_DIFF_H_
#define AUTOMATON_DIFF_H_

#include <ostream>

#include <automaton/Automaton.h>
#include <set>
#include <list>
#include <map>
#include <utility>

class AutomatonDiff {
	static bool testDiff(const automaton::UnknownAutomaton& a, const automaton::UnknownAutomaton& b);
	static void printDiff(const automaton::UnknownAutomaton& a, const automaton::UnknownAutomaton& b);

	static bool testDiff(const automaton::DFA& a, const automaton::DFA& b);
	static void printDiff(const automaton::DFA& a, const automaton::DFA& b);

	static bool testDiff(const automaton::NFA& a, const automaton::NFA& b);
	static void printDiff(const automaton::NFA& a, const automaton::NFA& b);

	static bool testDiff(const automaton::EpsilonNFA& a, const automaton::EpsilonNFA& b);
	static void printDiff(const automaton::EpsilonNFA& a, const automaton::EpsilonNFA& b);

	static bool testDiff(const automaton::ExtendedNFA& a, const automaton::ExtendedNFA& b);
	static void printDiff(const automaton::ExtendedNFA& a, const automaton::ExtendedNFA& b);

	static bool testDiff(const automaton::CompactNFA& a, const automaton::CompactNFA& b);
	static void printDiff(const automaton::CompactNFA& a, const automaton::CompactNFA& b);

	static bool testDiff(const automaton::NPDA& a, const automaton::NPDA& b);
	static void printDiff(const automaton::NPDA& a, const automaton::NPDA& b);

	static bool testDiff(const automaton::SinglePopNPDA& a, const automaton::SinglePopNPDA& b);
	static void printDiff(const automaton::SinglePopNPDA& a, const automaton::SinglePopNPDA& b);

	static bool testDiff(const automaton::OneTapeDTM& a, const automaton::OneTapeDTM& b);
	static void printDiff(const automaton::OneTapeDTM& a, const automaton::OneTapeDTM& b);

	template <class T> static void setDiff(const std::set<T> a, const std::set<T> b);
	template <class T> static void listDiff(const std::list<T> a, const std::list<T> b);
	template <class T, class R> static void mapDiff(const std::map<T, R> a, const std::map<T, R> b);
public:
	static int diff(const automaton::UnknownAutomaton& a, const automaton::UnknownAutomaton& b);
	static int diff(const automaton::DFA& a, const automaton::DFA& b);
	static int diff(const automaton::NFA& a, const automaton::NFA& b);
	static int diff(const automaton::EpsilonNFA& a, const automaton::EpsilonNFA& b);
	static int diff(const automaton::ExtendedNFA& a, const automaton::ExtendedNFA& b);
	static int diff(const automaton::CompactNFA& a, const automaton::CompactNFA& b);
	static int diff(const automaton::NPDA& a, const automaton::NPDA& b);
	static int diff(const automaton::SinglePopNPDA& a, const automaton::SinglePopNPDA& b);
	static int diff(const automaton::OneTapeDTM& a, const automaton::OneTapeDTM& b);

	template<typename T>
	static int dynamicDiff(const automaton::Automaton& a, const automaton::Automaton& b);

	static int diff(const automaton::Automaton& a, const automaton::Automaton& b);
};

#endif /* AUTOMATON_DIFF_H_ */
