//============================================================================
// Name		: aepislon.cpp
// Author	: Tomas Pecka
//============================================================================


#include <iostream>

#include <factory/DataFactory.hpp>
#include <exception/AlibException.h>

#include "epsilon/fsm/FSMEpsilonRemover.h"

int main(int argc, char** argv) {
	int fileParameterIndex = -1;

	try {
		if( argc > 1 )
		{
			for( int i = 1; i < argc; i++ )
			{
				if( std::string( "-h" ).compare( argv[i] ) == 0 )
				{
					std::cout << "Removes eps transitions from NFA." << std::endl;
					std::cout << "Usage: aepsilon [automaton.xml]" << std::endl;
					return 1;
				}
				else
				{
					if(fileParameterIndex == -1)
						fileParameterIndex = i;
					else
						throw exception::AlibException("Only one file can be passed as parameter - " + std::string(argv[i]) + " " + std::string(argv[fileParameterIndex]));
				}
			}
		}

		std::list<sax::Token> tokens;

		if(fileParameterIndex != -1) {
			sax::SaxParseInterface::parseFile(argv[fileParameterIndex], tokens);
		} else {
			sax::SaxParseInterface::parseStdin(tokens);
		}

		if(alib::FromXMLParsers::automatonParser.first(tokens)) {
			std::string xmlMark = tokens.front( ).getData( );
			automaton::Automaton automaton = alib::DataFactory::fromTokens<automaton::Automaton>(tokens);
			automaton::Automaton res = epsilon::FSMEpsilonRemover::remove( automaton );
			alib::DataFactory::toStdout(res);
		} else {
			throw exception::AlibException("Invalid argument expected Epsilon NFA.");
		}

	} catch (const exception::AlibException& exception) {
		alib::DataFactory::toStdout(exception);
		return 1;
	} catch (...) {
		return 127;
	}
}
