#ifndef __FATOLRGCONVERTER_H__
#define __FATOLRGCONVERTER_H__

#include <grammar/Regular/LeftRG.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

#include <grammar/Grammar.h>
#include <automaton/Automaton.h>

namespace fa2rg
{

/**
 * Finite automaton to right regular grammar converter.
 * Source: My own :)
 */
class FAtoLRGConverter : public automaton::VisitableAutomatonBase::const_visitor_type
{
public:
	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	static grammar::Grammar convert(const automaton::Automaton& automaton);

	static grammar::LeftRG convert(const automaton::NFA& automaton);
	static grammar::LeftRG convert(const automaton::DFA& automaton);

private:
	void Visit(void*, const automaton::UnknownAutomaton& automaton) const;
	void Visit(void*, const automaton::EpsilonNFA& automaton) const;
	void Visit(void*, const automaton::NFA& automaton) const;
	void Visit(void*, const automaton::DFA& automaton) const;
	void Visit(void*, const automaton::ExtendedNFA& automaton) const;
	void Visit(void*, const automaton::CompactNFA& automaton) const;
	void Visit(void*, const automaton::NPDA& automaton) const;
	void Visit(void*, const automaton::SinglePopNPDA& automaton) const;
	void Visit(void*, const automaton::DPDA& automaton) const;
	void Visit(void*, const automaton::SinglePopDPDA& automaton) const;
	void Visit(void*, const automaton::OneTapeDTM& automaton) const;

	static const FAtoLRGConverter FA_TO_LRG_CONVERTER;
};

} /* namespace fa2rg */

#endif /* __FATOLRGCONVERTER_H__ */
