/*
 * Brzozowski.cpp
 *
 *  Created on: 11. 1. 2014
 *      Author: tomas
 */

#include "Brzozowski.h"

#include <set>
#include <deque>
#include <queue>
#include <vector>

#include <string/LinearString.h>
#include <std/hexavigesimal.h>
#include <label/StringLabel.h>

#include "../../regexp/RegExpDerivation.h"
#include "../../regexp/RegExpOptimize.h"

namespace re2fa
{

Brzozowski::Brzozowski(void){}
Brzozowski::~Brzozowski(void){}


void Brzozowski::Visit(void* userData, const regexp::FormalRegExp& regexp)
{
    std::pair<std::set<alphabet::Symbol>, bool>& out = *(std::pair<std::set<alphabet::Symbol>, bool>*) userData;
    out.first = regexp.getAlphabet();
    out.second = regexp.containsEmptyString();
}
void Brzozowski::Visit(void* userData, const regexp::UnboundedRegExp& regexp)
{
    std::pair<std::set<alphabet::Symbol>, bool>& out = *(std::pair<std::set<alphabet::Symbol>, bool>*) userData;
    out.first = regexp.getAlphabet();
    out.second = regexp.containsEmptyString();
}

automaton::DFA Brzozowski::convert(const regexp::RegExp& regexp)
{
    // 1.
    regexp::RegExpOptimize opt;
    regexp::RegExp V = opt.optimize(regexp);

    std::pair<std::set<alphabet::Symbol>, bool> out({}, false);
    regexp.getData().Accept((void*) &out, *this);
    const std::set<alphabet::Symbol>& alphabet = out.first;

    std::set<regexp::RegExp> Q = { V };
    std::deque<std::set<regexp::RegExp>> Qi;

    Qi.push_back(std::set<regexp::RegExp>());
    Qi.at(0).insert(V);

    int i = 1;

    // 2.
    while(! Qi.at(i - 1).empty())
    {
        Qi.push_back(std::set<regexp::RegExp>()); // initialize set Q_i

        for(const auto& dregexp : Qi.at(i - 1))
        {
            regexp::RegExpDerivation deriv;

            for(const auto& a : alphabet)
            {
                string::LinearString string(std::vector<alphabet::Symbol>{a});
                regexp::RegExp derived = deriv.derivation(dregexp, string);
                derived = opt.optimize(derived);

                // this will also add \emptyset as a regexp (and as FA state)
                if(Q.count(derived) == 0) // if this state has already been found, do not add
                    Qi.at(i).insert(derived);
            }
        }

        Q.insert(Qi.at(i).begin(), Qi.at(i).end());
        i += 1;
    }

    std::map<regexp::RegExp, automaton::State> stateMap;
    int stateId = 0;

    for(const auto& r : Q)
    {
        automaton::State q(label::Label(label::StringLabel(std::toBase26(stateId++))));
        stateMap.insert(std::make_pair(r, q));
    }

    // ------------------------------------------------------------------------
    // 3.

    automaton::DFA automaton(stateMap.find(V)->second);

    for(const auto& r : stateMap)
    {
        automaton.addState(r.second);
    }

    automaton.setInputSymbols(alphabet);

    for(const auto& r : Q)
    {
        regexp::RegExpDerivation deriv;

        for(const auto& a: alphabet)
        {
            string::LinearString string(std::vector<alphabet::Symbol>{a});
            regexp::RegExp derived = deriv.derivation(r, string);
            derived = opt.optimize(derived);

            automaton.addTransition(stateMap.find(r)->second, a, stateMap.find(derived)->second);
        }
    }

    for(const auto& r : Q)
    {
        r.getData().Accept((void*) &out, *this);
        if(out.second) // if(r.containsEmptyString())
            automaton.addFinalState(stateMap.find(r)->second);
    }

    return automaton;
}

} /* namespace re2fa */
