/*
 * Glushkov.h
 *
 *  Created on: 11. 1. 2014
 *      Author: Tomas Pecka
 */

#ifndef GLUSHKOV_H_
#define GLUSHKOV_H_

#include <map>

#include <alphabet/Symbol.h>
#include <automaton/common/State.h>
#include <regexp/unbounded/UnboundedRegExp.h>
#include <regexp/unbounded/UnboundedRegExpElements.h>
#include <regexp/formal/FormalRegExp.h>
#include <regexp/formal/FormalRegExpElements.h>
#include <regexp/RegExp.h>

#include "../../regexp/GlushkovTraversal.h"
#include "std/hexavigesimal.h"

#include <automaton/FSM/NFA.h>

namespace re2fa {

/**
 * Converts regular expression to finite automaton using Glushkov's NFA construction algorithm.
 * Source: Melichar 2.107
 */
class Glushkov : public regexp::VisitableRegExpBase::visitor_type {
public:
    /**
     * Performs conversion.
     * @param re Original regular expression.
     * @return FSM equivalent to original regular expression.
     *
     */
    automaton::NFA convert( const regexp::UnboundedRegExp & re );
    automaton::NFA convert( const regexp::FormalRegExp & re );

    automaton::NFA convert( const regexp::RegExp & re );

private:
    std::map<regexp::GlushkovSymbol, automaton::State> m_stateMap;
    std::set<regexp::GlushkovSymbol> m_first;
    std::set<regexp::GlushkovSymbol> m_last;
    std::set<regexp::GlushkovPair> m_pairs;

    void Visit(void* , const regexp::FormalRegExp& regexp);
    void Visit(void* , const regexp::UnboundedRegExp& regexp);

};

} /* namespace conversions */

#endif /* GLUSHKOV_H_ */
