/*
 * Thompson.cpp
 *
 *  Created on: 11. 1. 2014
 *      Author: tomas
 */
#include "Thompson.h"
#include <tuple>
#include <label/Label.h>
#include <label/IntegerLabel.h>

namespace re2fa
{

Thompson::Thompson(void){}
Thompson::~Thompson(void){}

automaton::EpsilonNFA Thompson::convert(const regexp::RegExp& regexp)
{
    std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> out(automaton::EpsilonNFA(), 0, nullptr, nullptr);
    automaton::EpsilonNFA& automaton = std::get<0>(out);

    regexp.getData().Accept((void*) &out, *this);

    automaton.setInitialStates({*std::get<2>(out)});
    automaton.setFinalStates(std::set<automaton::State>{*std::get<3>(out)});

    return std::get<0>(out);
}

void Thompson::Visit(void* userData, const regexp::FormalRegExp& regexp)
{
    std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;
    automaton::EpsilonNFA& automaton = std::get<0>(out);

    automaton.setInputSymbols(regexp.getAlphabet());
    regexp.getRegExp().Accept((void*) &out, *this);
}

void Thompson::Visit(void* userData, const regexp::FormalRegExpAlternation& alternation)
{
    std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;
    automaton::EpsilonNFA& automaton = std::get<0>(out);

    automaton::State head = automaton::State(label::Label(label::IntegerLabel(std::get<1>(out)++)));
    automaton::State tail = automaton::State(label::Label(label::IntegerLabel(std::get<1>(out)++)));
    automaton.addState(head);
    automaton.addState(tail);

    static_cast<const regexp::FormalRegExpElement&>(alternation.getLeftElement()).Accept(userData, *this);
    automaton.addTransition(head, string::Epsilon::EPSILON, *std::get<2>(out));
    automaton.addTransition(*std::get<3>(out), string::Epsilon::EPSILON, tail);

    static_cast<const regexp::FormalRegExpElement&>(alternation.getRightElement()).Accept(userData, *this);
    automaton.addTransition(head, string::Epsilon::EPSILON, *std::get<2>(out));
    automaton.addTransition(*std::get<3>(out), string::Epsilon::EPSILON, tail);

    std::get<2>(out) = &(*automaton.getStates().find(head));
    std::get<3>(out) = &(*automaton.getStates().find(tail));
}

void Thompson::Visit(void* userData, const regexp::FormalRegExpConcatenation& concatenation)
{
    std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;

    automaton::EpsilonNFA& automaton = std::get<0>(out);

    static_cast<const regexp::FormalRegExpElement&>(concatenation.getLeftElement()).Accept(userData, *this);
    const automaton::State* leftHead = std::get<2>(out);
    const automaton::State* leftTail = std::get<3>(out);

    static_cast<const regexp::FormalRegExpElement&>(concatenation.getRightElement()).Accept(userData, *this);
    automaton.addTransition(*leftTail, string::Epsilon::EPSILON, *std::get<2>(out));

    std::get<2>(out) = &(*automaton.getStates().find(*leftHead));
    // std::get<3>(out) = std::get<3>(out);
}

void Thompson::Visit(void* userData, const regexp::FormalRegExpIteration& iteration)
{
    std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;

    automaton::EpsilonNFA& automaton = std::get<0>(out);

    automaton::State head = automaton::State(label::Label(label::IntegerLabel(std::get<1>(out)++)));
    automaton::State tail = automaton::State(label::Label(label::IntegerLabel(std::get<1>(out)++)));
    automaton.addState(head);
    automaton.addState(tail);

    static_cast<const regexp::FormalRegExpElement&>(iteration.getElement()).Accept(userData, *this);
    automaton.addTransition(head, string::Epsilon::EPSILON, *std::get<2>(out));
    automaton.addTransition(head, string::Epsilon::EPSILON, tail);
    automaton.addTransition(*std::get<3>(out), string::Epsilon::EPSILON, tail);
    automaton.addTransition(*std::get<3>(out), string::Epsilon::EPSILON, *std::get<2>(out));

    std::get<2>(out) = &(*automaton.getStates().find(head));
    std::get<3>(out) = &(*automaton.getStates().find(tail));
}

void Thompson::Visit(void* userData, const regexp::FormalRegExpSymbol& symbol)
{
    std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;

    automaton::EpsilonNFA& automaton = std::get<0>(out);

    automaton::State head = automaton::State(label::Label(label::IntegerLabel(std::get<1>(out)++)));
    automaton::State tail = automaton::State(label::Label(label::IntegerLabel(std::get<1>(out)++)));
    automaton.addState(head);
    automaton.addState(tail);

    automaton.addTransition(head, symbol.getSymbol(), tail);
    std::get<2>(out) = &(*automaton.getStates().find(head));
    std::get<3>(out) = &(*automaton.getStates().find(tail));
}

void Thompson::Visit(void* userData, const regexp::FormalRegExpEpsilon& epsilon)
{
    std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;

    automaton::EpsilonNFA& automaton = std::get<0>(out);

    automaton::State head = automaton::State(label::Label(label::IntegerLabel(std::get<1>(out)++)));
    automaton::State tail = automaton::State(label::Label(label::IntegerLabel(std::get<1>(out)++)));
    automaton.addState(head);
    automaton.addState(tail);

    automaton.addTransition(head, string::Epsilon::EPSILON, tail);
    std::get<2>(out) = &(*automaton.getStates().find(head));
    std::get<3>(out) = &(*automaton.getStates().find(tail));
}

void Thompson::Visit(void* userData, const regexp::FormalRegExpEmpty& empty)
{
    std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;

    automaton::EpsilonNFA& automaton = std::get<0>(out);

    automaton::State head = automaton::State(label::Label(label::IntegerLabel(std::get<1>(out)++)));

    automaton::State tail = automaton::State(label::Label(label::IntegerLabel(std::get<1>(out)++)));
    automaton.addState(head);
    automaton.addState(tail);

    std::get<2>(out) = &(*automaton.getStates().find(head));
    std::get<3>(out) = &(*automaton.getStates().find(tail));
}

void Thompson::Visit(void* userData, const regexp::UnboundedRegExp& regexp)
{
    std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;
    automaton::EpsilonNFA& automaton = std::get<0>(out);

    automaton.setInputSymbols(regexp.getAlphabet());
    regexp.getRegExp().Accept((void*) &out, *this);
}

void Thompson::Visit(void* userData, const regexp::UnboundedRegExpAlternation& alternation)
{
    std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;
    automaton::EpsilonNFA& automaton = std::get<0>(out);

    automaton::State head = automaton::State(label::Label(label::IntegerLabel(std::get<1>(out)++)));
    automaton::State tail = automaton::State(label::Label(label::IntegerLabel(std::get<1>(out)++)));
    automaton.addState(head);
    automaton.addState(tail);

    for(const auto& element : alternation.getElements())
    {
        static_cast<const regexp::UnboundedRegExpElement&>(*element).Accept(userData, *this);
        automaton.addTransition(head, string::Epsilon::EPSILON, *std::get<2>(out));
        automaton.addTransition(*std::get<3>(out), string::Epsilon::EPSILON, tail);
    }

    std::get<2>(out) = &(*automaton.getStates().find(head));
    std::get<3>(out) = &(*automaton.getStates().find(tail));
}

void Thompson::Visit(void* userData, const regexp::UnboundedRegExpConcatenation& concatenation)
{
    std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;
    automaton::EpsilonNFA& automaton = std::get<0>(out);

    std::vector<std::pair<const automaton::State*, const automaton::State*>> tails;
    for(const auto& element : concatenation.getElements())
    {
        static_cast<const regexp::UnboundedRegExpElement&>(*element).Accept(userData, *this);
        tails.push_back(std::make_pair(std::get<2>(out), std::get<3>(out)));
    }

    for(size_t i = 1; i < tails.size(); i++)
        automaton.addTransition(*tails[i-1].second, string::Epsilon::EPSILON, *tails[i].first);

    std::get<2>(out) = tails[0].first;
    std::get<3>(out) = tails[tails.size()-1].second;
}

void Thompson::Visit(void* userData, const regexp::UnboundedRegExpIteration& iteration)
{
    std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;
    automaton::EpsilonNFA& automaton = std::get<0>(out);

    automaton::State head = automaton::State(label::Label(label::IntegerLabel(std::get<1>(out)++)));
    automaton::State tail = automaton::State(label::Label(label::IntegerLabel(std::get<1>(out)++)));
    automaton.addState(head);
    automaton.addState(tail);

    static_cast<const regexp::UnboundedRegExpElement&>(iteration.getElement()).Accept(userData, *this);
    automaton.addTransition(head, string::Epsilon::EPSILON, *std::get<2>(out));
    automaton.addTransition(head, string::Epsilon::EPSILON, tail);
    automaton.addTransition(*std::get<3>(out), string::Epsilon::EPSILON, tail);
    automaton.addTransition(*std::get<3>(out), string::Epsilon::EPSILON, *std::get<2>(out));

    std::get<2>(out) = &(*automaton.getStates().find(head));
    std::get<3>(out) = &(*automaton.getStates().find(tail));
}

void Thompson::Visit(void* userData, const regexp::UnboundedRegExpSymbol& symbol)
{
    std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;
    automaton::EpsilonNFA& automaton = std::get<0>(out);

    automaton::State head = automaton::State(label::Label(label::IntegerLabel(std::get<1>(out)++)));
    automaton::State tail = automaton::State(label::Label(label::IntegerLabel(std::get<1>(out)++)));
    automaton.addState(head);
    automaton.addState(tail);

    automaton.addTransition(head, symbol.getSymbol(), tail);
    std::get<2>(out) = &(*automaton.getStates().find(head));
    std::get<3>(out) = &(*automaton.getStates().find(tail));
}

void Thompson::Visit(void* userData, const regexp::UnboundedRegExpEpsilon& epsilon)
{
    std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;
    automaton::EpsilonNFA& automaton = std::get<0>(out);

    automaton::State head = automaton::State(label::Label(label::IntegerLabel(std::get<1>(out)++)));
    automaton::State tail = automaton::State(label::Label(label::IntegerLabel(std::get<1>(out)++)));
    automaton.addState(head);
    automaton.addState(tail);

    automaton.addTransition(head, string::Epsilon::EPSILON, tail);
    std::get<2>(out) = &(*automaton.getStates().find(head));
    std::get<3>(out) = &(*automaton.getStates().find(tail));
}

void Thompson::Visit(void* userData, const regexp::UnboundedRegExpEmpty& empty)
{
    std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;
    automaton::EpsilonNFA& automaton = std::get<0>(out);

    automaton::State head = automaton::State(label::Label(label::IntegerLabel(std::get<1>(out)++)));
    automaton::State tail = automaton::State(label::Label(label::IntegerLabel(std::get<1>(out)++)));
    automaton.addState(head);
    automaton.addState(tail);

    std::get<2>(out) = &(*automaton.getStates().find(head));
    std::get<3>(out) = &(*automaton.getStates().find(tail));
}

} /* namespace re2fa */
