/*
 * Thompson.h
 *
 *  Created on: 11. 1. 2014
 *      Author: tomas
 */

#ifndef THOMPSON_H_
#define THOMPSON_H_

#include <regexp/RegExp.h>
#include <regexp/formal/FormalRegExpElements.h>
#include <regexp/unbounded/UnboundedRegExpElements.h>
#include <automaton/FSM/EpsilonNFA.h>

namespace re2fa
{

/**
 * Converts regular expression to finite automaton using Thompson's Construction Algorithm (TCA).
 * Sources:
 *  Hopcroft, section 3.2.3
 *  http://www.eecis.udel.edu/~cavazos/cisc672/lectures/Lecture-04.pdf
 *  http://citeseerx.ist.psu.edu/viewdoc/download?doi=10.1.1.21.7450&rep=rep1&type=ps
 *  Melichar 2.112
 */
class Thompson : public regexp::VisitableRegExpBase::visitor_type, regexp::FormalRegExpElement::visitor_type, regexp::UnboundedRegExpElement::visitor_type
{
public:
    /**
     * Performs conversion.
     * @return nondeterministic finite automaton with epsilon transitions accepting language described by the regexp
     */
    Thompson(void);
    ~Thompson(void);
    automaton::EpsilonNFA convert(const regexp::RegExp& regexp);

private:
     void Visit(void*, const regexp::UnboundedRegExp& regexp);
     void Visit(void*, const regexp::FormalRegExp& regexp);

     void Visit(void*, const regexp::UnboundedRegExpAlternation& alternation);
     void Visit(void*, const regexp::UnboundedRegExpConcatenation& concatenation);
     void Visit(void*, const regexp::UnboundedRegExpIteration& iteration);
     void Visit(void*, const regexp::UnboundedRegExpSymbol& symbol);
     void Visit(void*, const regexp::UnboundedRegExpEpsilon& epsilon);
     void Visit(void*, const regexp::UnboundedRegExpEmpty& empty);

     void Visit(void*, const regexp::FormalRegExpAlternation& alternation);
     void Visit(void*, const regexp::FormalRegExpConcatenation& concatenation);
     void Visit(void*, const regexp::FormalRegExpIteration& iteration);
     void Visit(void*, const regexp::FormalRegExpSymbol& symbol);
     void Visit(void*, const regexp::FormalRegExpEpsilon& epsilon);
     void Visit(void*, const regexp::FormalRegExpEmpty& empty);
};

} /* namespace re2fa */

#endif /* THOMPSON_H_ */
