/*
 * BrzozowskiDerivationRRG.cpp
 *
 *  Created on: 6. 3. 2014
 *      Author: tomas
 */

#include "BrzozowskiDerivationRRG.h"

#include <set>
#include <deque>
#include <set>
#include <vector>

#include <label/StringLabel.h>
#include <std/hexavigesimal.h>

#include "../../../regexp/RegExpOptimize.h"
#include "../../../regexp/RegExpDerivation.h"

namespace re2rg
{

BrzozowskiDerivationRRG::BrzozowskiDerivationRRG(void){}
BrzozowskiDerivationRRG::~BrzozowskiDerivationRRG(void){}


void BrzozowskiDerivationRRG::Visit(void* userData, const regexp::FormalRegExp& regexp)
{
    std::pair<std::set<alphabet::Symbol>, bool>& out = *(std::pair<std::set<alphabet::Symbol>, bool>*) userData;
    out.first = regexp.getAlphabet();
    out.second = regexp.containsEmptyString();
}
void BrzozowskiDerivationRRG::Visit(void* userData, const regexp::UnboundedRegExp& regexp)
{
    std::pair<std::set<alphabet::Symbol>, bool>& out = *(std::pair<std::set<alphabet::Symbol>, bool>*) userData;
    out.first = regexp.getAlphabet();
    out.second = regexp.containsEmptyString();
}

grammar::RightRG BrzozowskiDerivationRRG::convert(const regexp::RegExp& regexp)
{
    // 1.
    // regexp::RegExpOptimize opt;
    // RegExp V = opt.optimize(regexp);
    regexp::RegExp V = regexp;

    std::pair<std::set<alphabet::Symbol>, bool> out({}, false);
    regexp.getData().Accept((void*) &out, *this);
    const std::set<alphabet::Symbol>& alphabet = out.first;

    std::set<regexp::RegExp> N = { V };
    std::deque<std::set<regexp::RegExp>> Ni;

    Ni.push_back(std::set<regexp::RegExp>());
    Ni.at(0).insert(V);

    int i = 1;

    // 2.
    while(! Ni.at(i - 1).empty())
    {
        Ni.push_back(std::set<regexp::RegExp>()); // initialize set Q_i

        for(const auto & dregexp : Ni.at( i - 1 ))
        {
            regexp::RegExpDerivation deriv;

            for(const auto & a : alphabet)
            {
                string::LinearString string(std::vector<alphabet::Symbol>{a});
                regexp::RegExp derived = deriv.derivation(dregexp, string);
                // derived = opt.optimize(derived);

                // this will also add \emptyset as a regexp (and as FA state)
                if(N.count(derived) == 0) // if this state has already been found, do not add
                    Ni.at(i).insert(derived);
            }
        }

        N.insert(Ni.at(i).begin(), Ni.at(i).end());
        i += 1;
    }

    // ------------------------------------------------------------------------
    // 3.

    int nonterminalId = 0;
    std::map<regexp::RegExp, alphabet::Symbol> nonterminalMap;

    alphabet::Symbol ntV(alphabet::LabeledSymbol(label::Label(label::StringLabel(std::toBase26(nonterminalId++)))));
    nonterminalMap.insert(std::make_pair(V, ntV));

    grammar::RightRG grammar(ntV);
    grammar.setTerminalAlphabet(alphabet);

    for(const auto & r : N)
    {
        if(V == r) continue;

        alphabet::Symbol nt = alphabet::createUniqueSymbol(alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel(std::toBase26(nonterminalId++))))), grammar.getTerminalAlphabet(), grammar.getNonterminalAlphabet());
        nonterminalMap.insert(std::make_pair(r, nt));
    }

    for(const auto & r : N)
    {
        regexp::RegExpDerivation deriv;

        for(const auto & a : alphabet)
        {
            string::LinearString string(std::vector<alphabet::Symbol>{a});
            regexp::RegExp derived = deriv.derivation(r, string);
            // derived = opt.optimize(derived);

            grammar.addRule(nonterminalMap.find(r)->second, std::make_pair(a, nonterminalMap.find(derived)->second));

            derived.getData().Accept((void*) &out, *this);
            if(out.second) // if(derived.containsEmptyString())
                grammar.addRule(nonterminalMap.find(r)->second, a);
        }
    }

    grammar.setInitialSymbol(nonterminalMap.find(V)->second);

    V.getData().Accept((void*) &out, *this);
    if(out.second) // if(V.containsEmptyString())
        grammar.setGeneratesEpsilon(true); // okay, because of this feature we do not have to bother with extending the grammar with new rules and nonterminals. YAY!

    return grammar;
}

} /* namespace re2rg */
