#include "LRGtoFAConverter.h"

#include <label/StringLabel.h>
#include <automaton/Automaton.h>
#include <alphabet/LabeledSymbol.h>

namespace rg2fa
{

automaton::NFA LRGtoFAConverter::convert(const grammar::LeftRG& grammar)
{
    std::map<alphabet::Symbol, automaton::State> stateMap;

    automaton::NFA automaton;
    automaton.setInputSymbols(grammar.getTerminalAlphabet());
    for(const auto& symbol : grammar.getNonterminalAlphabet())
    {
        automaton::State state(static_cast<const alphabet::LabeledSymbol&>(symbol.getData()).getLabel());
        automaton.addState(state);
        stateMap.insert(std::make_pair(symbol, state));
    }

    const automaton::State startState = automaton::createUniqueState(automaton::State(label::Label(label::StringLabel("q0"))), automaton.getStates());
    automaton.addState(startState);

    // step 3, constructing \delta
    for(const auto& rule : grammar.getRules())
    {
        const alphabet::Symbol& lhs = rule.first;
        for(const auto& ruleRHS : rule.second)
        {
            if(ruleRHS.is<std::pair<alphabet::Symbol, alphabet::Symbol>>()) // if B->Ca => \delta(C,a)=B
            {
                const std::pair<alphabet::Symbol, alphabet::Symbol>& rhs = ruleRHS.get<std::pair<alphabet::Symbol, alphabet::Symbol>>();
                automaton.addTransition(stateMap.find(rhs.first)->second, rhs.second, stateMap.find(lhs)->second);
            }
            else // if B->a => \delta(StartState,a)=B
            {
                const alphabet::Symbol& rhs = ruleRHS.get<alphabet::Symbol>();
                automaton.addTransition(startState, rhs, stateMap.find(lhs)->second);
            }
        }
    }

    // step 4
    automaton.addInitialState(startState);

    // step 5
    automaton.addFinalState(stateMap.find(grammar.getInitialSymbol())->second);
    if(grammar.getGeneratesEpsilon())
        automaton.addFinalState(startState);

    return automaton;
}

} /* namespace rg2fa */
