#include "RRGtoFAConverter.h"

#include <label/StringLabel.h>
#include <automaton/Automaton.h>
#include <alphabet/LabeledSymbol.h>

namespace rg2fa
{

automaton::NFA RRGtoFAConverter::convert(const grammar::RightRG& grammar)
{
    std::map<alphabet::Symbol, automaton::State> stateMap;

    automaton::NFA automaton;
    automaton.setInputSymbols(grammar.getTerminalAlphabet());
    for(const auto& symbol : grammar.getNonterminalAlphabet())
    {
        automaton::State state(static_cast<const alphabet::LabeledSymbol&>(symbol.getData()).getLabel());
        automaton.addState(state);
        stateMap.insert(std::make_pair(symbol, state));
    }

    const automaton::State AState = automaton::createUniqueState(automaton::State(label::Label(label::StringLabel("A"))), automaton.getStates());
    automaton.addState(AState);

    // step 3, construct \delta
    for(const auto& rule : grammar.getRules())
    {
        const alphabet::Symbol& lhs = rule.first;
        for(const auto& ruleRHS : rule.second)
        {
            if(ruleRHS.is<std::pair<alphabet::Symbol, alphabet::Symbol>>()) // if B->aC => \delta(B,a)=C
            {
                const std::pair<alphabet::Symbol, alphabet::Symbol>& rhs = ruleRHS.get<std::pair<alphabet::Symbol, alphabet::Symbol>>();
                automaton.addTransition(stateMap.find(lhs)->second, rhs.first, stateMap.find(rhs.second)->second);
            }
            else // if B->a => \delta(B,a)=AState
            {
                const alphabet::Symbol& rhs = ruleRHS.get<alphabet::Symbol>();
                automaton.addTransition(stateMap.find(lhs)->second, rhs, AState);
            }
        }
    }

    // step 4
    automaton.addInitialState(stateMap.find(grammar.getInitialSymbol())->second);

    // step 5
    automaton.addFinalState(AState);
    if(grammar.getGeneratesEpsilon())
        for(const auto& state : automaton.getInitialStates())
            automaton.addFinalState(state);

    return automaton;
}

} /* namespace conversions */
