/*
 * LeftToRightRegularGrammar.cpp
 *
 *  Created on: 8. 3. 2014
 *      Author: Tomas Pecka
 */

#include "LeftToRightRegularGrammar.h"

namespace rg2rg {

grammar::RightRG LeftToRightRegularGrammar::convert(const grammar::LeftRG& lrg)
{
	// 1.
	alphabet::Symbol s = alphabet::createUniqueSymbol( lrg.getInitialSymbol( ), lrg.getNonterminalAlphabet(), lrg.getTerminalAlphabet() );

	grammar::RightRG rrg( s );

	for(const auto & nonterminalSymbol : lrg.getNonterminalAlphabet() ) {
		rrg.addNonterminalSymbol( nonterminalSymbol );
	}

	rrg.setTerminalAlphabet( lrg.getTerminalAlphabet( ) );
	rrg.setGeneratesEpsilon( lrg.getGeneratesEpsilon( ) );

	// 2
	for( const auto & rule : lrg.getRules( ) ) {
		const alphabet::Symbol& lhs = rule.first;

		for(const auto & ruleRHS : rule.second ) {
		        if( ruleRHS.is<std::pair<alphabet::Symbol, alphabet::Symbol>>( ) ) {
				const std::pair<alphabet::Symbol, alphabet::Symbol>& rhs = ruleRHS.get<std::pair<alphabet::Symbol, alphabet::Symbol>>();

				alphabet::Symbol leftSide = ( rhs.first );
				std::pair<alphabet::Symbol, alphabet::Symbol> rightSide = std::make_pair( rhs.second, lhs );
				rrg.addRule( leftSide, rightSide );

				if( lhs == lrg.getInitialSymbol( ) ) {
					alphabet::Symbol leftSide = rhs.first;
					alphabet::Symbol rightSide = rhs.second;
					rrg.addRule( leftSide, rightSide );
				}
			} else {
				const alphabet::Symbol& rhs = ruleRHS.get<alphabet::Symbol>();

				alphabet::Symbol leftSide = rrg.getInitialSymbol( );
				std::pair<alphabet::Symbol, alphabet::Symbol> rightSide = std::make_pair( rhs, lhs );
				rrg.addRule( leftSide, rightSide );

				if( lhs == lrg.getInitialSymbol( ) ) {
					alphabet::Symbol leftSide = rrg.getInitialSymbol( );
					alphabet::Symbol rightSide = rhs;
					rrg.addRule( leftSide, rightSide );
				}
			}
		}
	}
	return rrg;
}

} /* namespace rg2rg */
