/*
 * RightToLeftRegularGrammar.cpp
 *
 *  Created on: 8. 3. 2014
 *      Author: Tomas Pecka
 */

#include "RightToLeftRegularGrammar.h"

namespace rg2rg
{

grammar::LeftRG RightToLeftRegularGrammar::convert(const grammar::RightRG& rrg)
{
	// 1.
	alphabet::Symbol s = alphabet::createUniqueSymbol( rrg.getInitialSymbol( ), rrg.getNonterminalAlphabet(), rrg.getTerminalAlphabet() );

	grammar::LeftRG lrg(s);

	for(const auto & nonterminalSymbol : rrg.getNonterminalAlphabet()) {
		lrg.addNonterminalSymbol( nonterminalSymbol );
	}

	lrg.setTerminalAlphabet( rrg.getTerminalAlphabet( ) );
	lrg.setGeneratesEpsilon( rrg.getGeneratesEpsilon( ) );

	// 2.
	for( const auto & rule : rrg.getRules( ) ) {
		const alphabet::Symbol& lhs = rule.first;

		for(const auto & ruleRHS : rule.second ) {
		        if( ruleRHS.is<std::pair<alphabet::Symbol, alphabet::Symbol>>( ) ) {
				const std::pair<alphabet::Symbol, alphabet::Symbol>& rhs = ruleRHS.get<std::pair<alphabet::Symbol, alphabet::Symbol>>();

				alphabet::Symbol leftSide = rhs.second;
				std::pair<alphabet::Symbol, alphabet::Symbol> rightSide = std::make_pair( lhs, rhs.first );
				lrg.addRule( leftSide, rightSide );

				if( lhs == rrg.getInitialSymbol( ) ) {
					alphabet::Symbol leftSide = rhs.second;
					alphabet::Symbol rightSide = rhs.first;
					lrg.addRule( leftSide, rightSide );
				}
			} else {
				const alphabet::Symbol& rhs = ruleRHS.get<alphabet::Symbol>();

				alphabet::Symbol leftSide = lrg.getInitialSymbol( );
				std::pair<alphabet::Symbol, alphabet::Symbol> rightSide = std::make_pair ( lhs, rhs );
				lrg.addRule( leftSide, rightSide );

				if( lhs == rrg.getInitialSymbol( ) ) {
					alphabet::Symbol leftSide = lrg.getInitialSymbol( );
					alphabet::Symbol rightSide = rhs;
					lrg.addRule( leftSide, rightSide );
				}
			}
		}
	}
	return lrg;
}

} /* namespace rg2rg */
