/*
 * NFADeterminizer.h
 *
 *  Created on: 16. 1. 2014
 *	  Author: Jan Vesely
 */

#ifndef NFA_DETERMINIZER_H_
#define NFA_DETERMINIZER_H_

#include <set>

#include "automaton/common/State.h"
#include "automaton/FSM/NFA.h"
#include "automaton/FSM/DFA.h"

namespace determinize {

/**
 * Class for running determinization algorithm on fsm.
 */
class NFADeterminizer {

private:

	/**
	 * Returns existing state from the resulting automaton, if there is one, or creates new one and adds it into
	 *  the resulting deterministic automaton.
	 *
	 * @param originalStates set of states from nondeterministic fsm which represents state of deterministic fsm
	 * @return state of deterministic fsm
	 */
	static automaton::State createDFAState(const std::set<automaton::State>& nfaStates);

	/**
	 * Finds states from nondeterministic fsm to which at least one state from given set of states have transition
	 *  with given input.
	 *
	 * @param fromStates set of states from nondeterministic fsm
	 * @param input symbol from input alphabet
	 * @return set of states from nondeterministic fsm
	 */
	static std::set<automaton::State> recreateNFAStates(const automaton::State& dfaState);
public:

	/**
	 * @param nfsm nondeterministic final-state machine given for determinization
	 * Runs determinization algorithm on nondeterministic fsm given in constructor.
	 */
	static automaton::DFA determinize(const automaton::NFA& nfa);

};

} /* namespace determinize */

#endif /* NFA_DETERMINIZER_H_ */
