/*
 * FSMEpsilonRemover.cpp
 *
 *  Created on: 16. 1. 2014
 *	  Author: Tomas Pecka
 */

#include "FSMEpsilonRemover.h"

#include "../../automaton/EpsilonClosure.h"

namespace epsilon {

automaton::DFA FSMEpsilonRemover::remove(const automaton::DFA& origFSM)
{
    return origFSM;
}

automaton::NFA FSMEpsilonRemover::remove(const automaton::NFA& origFSM)
{
    return origFSM;
}

automaton::NFA FSMEpsilonRemover::remove( const automaton::EpsilonNFA & origFSM ) {
	automaton::NFA fsm;

	for( const auto & state : origFSM.getStates() )
		fsm.addState( state );

	for( const auto & state : origFSM.getInitialStates() )
		fsm.addInitialState( state );

	for( const auto & symbol : origFSM.getInputAlphabet() )
		fsm.addInputSymbol( symbol );

	/**
	 * Step 1 from Melichar 2.41
	 */
	for( const auto & from : origFSM.getStates( ) )
		for( const auto & fromClosure : automaton::EpsilonClosure::epsilonClosure( origFSM, from ) )
			for( const auto & transition : origFSM.getSymbolTransitionsFromState( fromClosure ) )
				for( const auto & to : transition.second )
					fsm.addTransition( from, transition.first.second, to );

	/**
	 * Step 2 from Melichar 2.41
	 */
	std::set<automaton::State> finalStates;

	for( const auto & q : fsm.getStates( ) )
	{
		const std::set<automaton::State> & cl = automaton::EpsilonClosure::epsilonClosure( origFSM, q );
		const std::set<automaton::State> & F = origFSM.getFinalStates( );
		std::set<automaton::State> intersect;

		set_intersection( cl.begin(), cl.end(), F.begin(), F.end(), std::inserter( intersect, intersect.begin() ) );
		if( intersect.size( ) != 0 )
			finalStates.insert( q );
	}

	for( const auto & q : finalStates )
		fsm.addFinalState( q );

	return fsm;
}

automaton::Automaton FSMEpsilonRemover::remove(const automaton::Automaton& automaton) {
	automaton::Automaton* out = NULL;
	automaton.getData().Accept((void*) &out, FSMEpsilonRemover::FSM_EPSILON_REMOVER);
	automaton::Automaton res = std::move(*out);
	delete out;
	return res;
}

void FSMEpsilonRemover::Visit(void*, const automaton::UnknownAutomaton&) const {
	throw exception::AlibException("Unsupported automaton type UnknownAutomaton");
}

void FSMEpsilonRemover::Visit(void* data, const automaton::EpsilonNFA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->remove(automaton));
}

void FSMEpsilonRemover::Visit(void* data, const automaton::NFA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->remove(automaton));
}

void FSMEpsilonRemover::Visit(void* data, const automaton::DFA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->remove(automaton));
}

void FSMEpsilonRemover::Visit(void*, const automaton::ExtendedNFA& ) const {
	throw exception::AlibException("Unsupported automaton type UnknownAutomaton");
}

void FSMEpsilonRemover::Visit(void*, const automaton::CompactNFA& ) const {
	throw exception::AlibException("Unsupported automaton type UnknownAutomaton");
}

void FSMEpsilonRemover::Visit(void*, const automaton::DPDA&) const {
	throw exception::AlibException("Unsupported automaton type UnknownAutomaton");
}

void FSMEpsilonRemover::Visit(void*, const automaton::SinglePopDPDA&) const {
	throw exception::AlibException("Unsupported automaton type UnknownAutomaton");
}

void FSMEpsilonRemover::Visit(void*, const automaton::NPDA&) const {
	throw exception::AlibException("Unsupported automaton type UnknownAutomaton");
}

void FSMEpsilonRemover::Visit(void*, const automaton::SinglePopNPDA&) const {
	throw exception::AlibException("Unsupported automaton type UnknownAutomaton");
}

void FSMEpsilonRemover::Visit(void*, const automaton::OneTapeDTM&) const {
	throw exception::AlibException("Unsupported automaton type UnknownAutomaton");
}

const FSMEpsilonRemover FSMEpsilonRemover::FSM_EPSILON_REMOVER;

} /* namespace epsilon */
