/*
 * LeftRegularEquationSolver.cpp
 *
 *  Created on: 4. 3. 2014
 *	  Author: Tomas Pecka
 */

#include "LeftRegularEquationSolver.h"
#include "regexp/unbounded/UnboundedRegExpElements.h"

#include "../regexp/RegExpOptimize.h"

namespace equations {

regexp::UnboundedRegExp LeftRegularEquationSolver::eliminate( void ) {
	regexp::RegExpOptimize opt;

	for( auto itA = m_symbolsByDepth.rbegin( ); itA != m_symbolsByDepth.rend( ); itA ++ ) {
		const alphabet::Symbol & a = * itA;

		/*
		 * Apply Arden's Lemma
		 * A = A0 + B1 + C2
		 * => A = 10*B + 20*C
		 */
		regexp::UnboundedRegExpIteration loop( std::move( m_eqTransition[ a ][ a ] ) );
		opt.optimize( loop );

		// for all transitions from A apply Arden's Lemma
		for( auto itB = std::next( itA ) ; itB != m_symbolsByDepth.rend( ); itB ++ ) {
			const alphabet::Symbol & b = * itB;
			regexp::UnboundedRegExpConcatenation concat;
			concat.appendElement( std::move( m_eqTransition[ a ][ b ] ) );
			concat.appendElement( loop );
			regexp::UnboundedRegExpAlternation alt;
			alt.appendElement( std::move( concat ) );
			m_eqTransition[ a ][ b ] = std::move( alt );
			opt.optimize( m_eqTransition[ a ][ b ] );
		}
		regexp::UnboundedRegExpConcatenation concat;
		concat.appendElement( std::move( m_eqFinal[ a ] ) );
		concat.appendElement( std::move( loop ) );
		regexp::UnboundedRegExpAlternation alt;
		alt.appendElement( std::move( concat ) );
		m_eqFinal[ a ] = std::move( alt );
		opt.optimize( m_eqFinal[ a ] );

		/*
		 * eliminate A from rest of the equations using this pattern:
		 * B->C = B->C + concatenate( B->A, A->C )
		 */
		for( auto itB = std::next( itA ); itB != m_symbolsByDepth.rend( ); itB ++ ) {
			const alphabet::Symbol & b = * itB;

			for( auto itC = std::next( itA ); itC != m_symbolsByDepth.rend( ); itC ++ ) {
				const alphabet::Symbol & c = * itC;

				regexp::UnboundedRegExpConcatenation concat;
				concat.appendElement( m_eqTransition[ a ][ c ] );
				concat.appendElement( m_eqTransition[ b ][ a ] );
				regexp::UnboundedRegExpAlternation alt;
				alt.appendElement( std::move( m_eqTransition[ b ][ c ] ) );
				alt.appendElement( std::move( concat ) );
				m_eqTransition[ b ][ c ] = std::move( alt );
				opt.optimize( m_eqTransition[ b ][ c ] );
			}

			regexp::UnboundedRegExpConcatenation concat;
			concat.appendElement( m_eqFinal[ a ] );
			concat.appendElement( std::move( m_eqTransition[ b ][ a ] ) );
			regexp::UnboundedRegExpAlternation alt;
			alt.appendElement( std::move( m_eqFinal[ b ] ) );
			alt.appendElement( std::move( concat ) );
			m_eqFinal[ b ] = std::move( alt );
			opt.optimize( m_eqFinal[ b ] );
		}
	}

	return opt.optimize( regexp::UnboundedRegExp( std::move ( m_eqFinal[ * m_symbolsByDepth.begin( ) ] ) ) );
}

} /* namespace equations */
