/*
 * GrammarPropertiesCFG.cpp
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#include "GrammarPropertiesCFG.h"

#include <grammar/ContextFree/CFG.h>
#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <grammar/ContextFree/GNF.h>
#include <grammar/ContextFree/CNF.h>
#include <grammar/ContextFree/LG.h>
#include <grammar/Regular/LeftLG.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightLG.h>
#include <grammar/Regular/RightRG.h>

#include <std/set.hpp>

namespace grammar {

template<class T>
std::set<alphabet::Symbol> GrammarPropertiesCFG::getProductiveNonterminals( const T & grammar ) {
	// 1.
	std::deque<std::set<alphabet::Symbol>> Ni;
	Ni.push_back( std::set<alphabet::Symbol>( ) );

	int i = 1;

	// 2.
	while( true ) {
		Ni.push_back( Ni.at( i - 1 ) );

		for( const auto & rule : grammar.getRawRules( ) ) {
			for( const auto & rhs : rule.second ) {
				if( std::all_of( rhs.begin( ), rhs.end( ), [ i, Ni, grammar ]( const alphabet::Symbol & symbol ) -> bool {
					return Ni.at( i - 1 ) . count( symbol ) || grammar.getTerminalAlphabet( ). count( symbol );
				} ) )
					Ni.at( i ).insert( rule.first );
			}
		}

		if( Ni.at( i ) == Ni.at( i - 1 ) )
			break;

		i = i + 1;
	}

	// 3.
	return Ni.at( i );
}

template std::set<alphabet::Symbol> GrammarPropertiesCFG::getProductiveNonterminals( const grammar::CFG & grammar );
template std::set<alphabet::Symbol> GrammarPropertiesCFG::getProductiveNonterminals( const grammar::EpsilonFreeCFG & grammar );
template std::set<alphabet::Symbol> GrammarPropertiesCFG::getProductiveNonterminals( const grammar::GNF & grammar );
template std::set<alphabet::Symbol> GrammarPropertiesCFG::getProductiveNonterminals( const grammar::CNF & grammar );
template std::set<alphabet::Symbol> GrammarPropertiesCFG::getProductiveNonterminals( const grammar::LG & grammar );
template std::set<alphabet::Symbol> GrammarPropertiesCFG::getProductiveNonterminals( const grammar::LeftLG & grammar );
template std::set<alphabet::Symbol> GrammarPropertiesCFG::getProductiveNonterminals( const grammar::LeftRG & grammar );
template std::set<alphabet::Symbol> GrammarPropertiesCFG::getProductiveNonterminals( const grammar::RightLG & grammar );
template std::set<alphabet::Symbol> GrammarPropertiesCFG::getProductiveNonterminals( const grammar::RightRG & grammar );

template<class T>
std::set<alphabet::Symbol> GrammarPropertiesCFG::getUnreachableSymbols( const T & grammar ) {
	// 1
	std::deque<std::set<alphabet::Symbol>> Vi;
	Vi.push_back( std::set<alphabet::Symbol>( ) );
	Vi.at( 0 ).insert( grammar.getInitialSymbol( ) );

	int i = 1;

	// 2.
	while( true ) {
		Vi.push_back( Vi.at( i - 1 ) );

		for( const auto & rule : grammar.getRawRules( ) ) {
			if( Vi.at( i - 1 ).count( rule.first ) ) {
				for( const auto & rhs : rule.second ) {
					Vi.at( i ).insert( rhs.begin( ), rhs.end( ) );
				}
			}
		}


		if( Vi.at( i ) == Vi.at( i - 1 ) )
			break;

		i = i + 1;
	}

	// 3.
	return Vi.at( i );
}

template std::set<alphabet::Symbol> GrammarPropertiesCFG::getUnreachableSymbols( const grammar::CFG & grammar );
template std::set<alphabet::Symbol> GrammarPropertiesCFG::getUnreachableSymbols( const grammar::EpsilonFreeCFG & grammar );
template std::set<alphabet::Symbol> GrammarPropertiesCFG::getUnreachableSymbols( const grammar::GNF & grammar );
template std::set<alphabet::Symbol> GrammarPropertiesCFG::getUnreachableSymbols( const grammar::CNF & grammar );
template std::set<alphabet::Symbol> GrammarPropertiesCFG::getUnreachableSymbols( const grammar::LG & grammar );
template std::set<alphabet::Symbol> GrammarPropertiesCFG::getUnreachableSymbols( const grammar::LeftLG & grammar );
template std::set<alphabet::Symbol> GrammarPropertiesCFG::getUnreachableSymbols( const grammar::LeftRG & grammar );
template std::set<alphabet::Symbol> GrammarPropertiesCFG::getUnreachableSymbols( const grammar::RightLG & grammar );
template std::set<alphabet::Symbol> GrammarPropertiesCFG::getUnreachableSymbols( const grammar::RightRG & grammar );

}

