/*
 * NormalizeDFA.cpp
 *
 *  Created on: Dec 9, 2013
 *      Author: JanTravnicek */

#include "NormalizeDFA.h"

#include <map>
#include <deque>
#include <vector>
#include <set>
#include <algorithm>
#include <sstream>
#include <iostream>
#include "exception/AlibException.h"

#include "alphabet/Symbol.h"
#include "label/Label.h"
#include "label/IntegerLabel.h"

#include "automaton/Automaton.h"

namespace normalize {

automaton::DFA NormalizeDFA::normalize(automaton::DFA& fsm) {
	int counter = 0;
	std::map<automaton::State, int > normalizationData;
	std::deque<automaton::State > processingData;
	
	normalizationData.insert(std::pair<automaton::State, int>(fsm.getInitialState(), counter++));
	processingData.push_back(fsm.getInitialState());
	
	while(!processingData.empty()) {
		automaton::State current = processingData.front();
		processingData.pop_front();
		
		std::map<std::pair<automaton::State, alphabet::Symbol>, automaton::State> transitionsMap = fsm.getTransitionsFromState(current);
		// Transitions are trivialy sorted by input symbol (from state is the same)
		
		for(auto iter = fsm.getTransitions().begin(); iter != fsm.getTransitions().end(); iter++) {
			if(normalizationData.find(iter->second) == normalizationData.end()) {
				normalizationData.insert(std::pair<automaton::State, int>(iter->second, counter++));
				processingData.push_back(iter->second);
			}
		}
	}
	
	if(normalizationData.size() != fsm.getStates().size()) {
		throw exception::AlibException("Automaton normalize require minimal deterministic finite automaton");
	}
	
	automaton::DFA result(automaton::stateFromInteger(normalizationData.find(fsm.getInitialState())->second));
	
	result.setInputSymbols(fsm.getInputAlphabet());
	
	for(auto iter = fsm.getStates().begin(); iter != fsm.getStates().end(); iter++) {
		result.addState(automaton::stateFromInteger(normalizationData.find(*iter)->second));
	}
	
	for(auto iter = fsm.getFinalStates().begin(); iter != fsm.getFinalStates().end(); iter++) {
		result.addFinalState(automaton::stateFromInteger(normalizationData.find(*iter)->second));
	}
	
	for(auto iter = fsm.getTransitions().begin(); iter != fsm.getTransitions().end(); iter++) {
		result.addTransition(automaton::stateFromInteger(normalizationData.find(iter->first.first)->second), iter->first.second, automaton::stateFromInteger(normalizationData.find(iter->second)->second));
	}

	return result;
}

}
