/*
 * RegExpIntegral.h
 *
 *  Created on: 24. 2. 2014
 *      Author: Tomas Pecka
 */

#ifndef REGEXPINTEGRAL_H_
#define REGEXPINTEGRAL_H_

#include <regexp/RegExp.h>
#include <regexp/formal/FormalRegExpElements.h>
#include <regexp/unbounded/UnboundedRegExpElements.h>

#include <string/LinearString.h>
#include <string/StringBase.h>

namespace regexp
{

/**
 * Calculates integral of regular expression as defined in Melichar 2.93 (chapt. 2.4.4)
 */
class RegExpIntegral    : public regexp::VisitableRegExpBase::visitor_type, regexp::FormalRegExpElement::visitor_type, regexp::UnboundedRegExpElement::visitor_type
{
public:
    RegExpIntegral();
    ~RegExpIntegral();
    regexp::RegExp integral(const regexp::RegExp& regexp, const string::LinearString& string);

private:
    void Visit(void*, const regexp::UnboundedRegExpAlternation& alternation);
    void Visit(void*, const regexp::UnboundedRegExpConcatenation& concatenation);
    void Visit(void*, const regexp::UnboundedRegExpIteration& iteration);
    void Visit(void*, const regexp::UnboundedRegExpSymbol& symbol);
    void Visit(void*, const regexp::UnboundedRegExpEpsilon& epsilon);
    void Visit(void*, const regexp::UnboundedRegExpEmpty& empty);

    void Visit(void*, const regexp::FormalRegExpAlternation& alternation);
    void Visit(void*, const regexp::FormalRegExpConcatenation& concatenation);
    void Visit(void*, const regexp::FormalRegExpIteration& iteration);
    void Visit(void*, const regexp::FormalRegExpSymbol& symbol);
    void Visit(void*, const regexp::FormalRegExpEpsilon& epsilon);
    void Visit(void*, const regexp::FormalRegExpEmpty& empty);

    void Visit(void*, const regexp::UnboundedRegExp& empty);
    void Visit(void*, const regexp::FormalRegExp& empty);
};

} /* namespace regexp */

#endif /* REGEXPINTEGRAL_H_ */
