#include <list>
#include "re2faTest.h"

#include "conversions/re2fa/Brzozowski.h"
#include "conversions/re2fa/Glushkov.h"
#include "conversions/re2fa/Thompson.h"
#include "conversions/fa2re/BrzozowskiAlgebraic.h"
#include "determinize/nfa/NFADeterminizer.h"
#include "minimize/dfa/MinimizeDFA.h"
#include "epsilon/fsm/FSMEpsilonRemover.h"

#include "regexp/unbounded/UnboundedRegExp.h"
#include "regexp/RegExpFromStringParser.h"

#include "automaton/FSM/NFA.h"

CPPUNIT_TEST_SUITE_REGISTRATION( re2faTest );

void re2faTest::setUp() {
}

void re2faTest::tearDown() {
}

void re2faTest::testThompson() {
	std::string input = "a+a* b*";
	std::stringstream inputs(input);

	regexp::RegExpFromStringParser parser(inputs);
	regexp::RegExp regexp1( parser.parseValue() );

	re2fa::Thompson thompson1;
	automaton::EpsilonNFA enfa1 = thompson1.convert(regexp1);

	regexp::RegExp regexp2( fa2re::BrzozowskiAlgebraic::convert(enfa1) );

	re2fa::Thompson thompson2;
	automaton::EpsilonNFA enfa2 = thompson2.convert(regexp2);

	automaton::NFA nfa1 = epsilon::FSMEpsilonRemover::remove(enfa1);
	automaton::NFA nfa2 = epsilon::FSMEpsilonRemover::remove(enfa2);

	automaton::DFA dfa1 = determinize::NFADeterminizer::determinize(nfa1);
	automaton::DFA dfa2 = determinize::NFADeterminizer::determinize(nfa2);

	automaton::DFA mdfa1 = minimize::MinimizeDFA::minimize(dfa1);
	automaton::DFA mdfa2 = minimize::MinimizeDFA::minimize(dfa2);

	CPPUNIT_ASSERT( mdfa1 == mdfa2);
}

void re2faTest::testGlushkov() {
	std::string input = "a+a* b*";
	std::stringstream inputs(input);

	regexp::RegExpFromStringParser parser(inputs);
	regexp::UnboundedRegExp regexp1( static_cast<const regexp::UnboundedRegExp &>( parser.parseValue().getData() ) );

	re2fa::Glushkov glushkov1;
	automaton::NFA nfa1 = glushkov1.convert(regexp1);

	regexp::UnboundedRegExp regexp2( static_cast<const regexp::UnboundedRegExp &>( fa2re::BrzozowskiAlgebraic::convert(nfa1) ) );

	re2fa::Glushkov glushkov2;
	automaton::NFA nfa2 = glushkov2.convert(regexp2);

	automaton::DFA dfa1 = determinize::NFADeterminizer::determinize(nfa1);
	automaton::DFA dfa2 = determinize::NFADeterminizer::determinize(nfa2);

	automaton::DFA mdfa1 = minimize::MinimizeDFA::minimize(dfa1);
	automaton::DFA mdfa2 = minimize::MinimizeDFA::minimize(dfa2);

	CPPUNIT_ASSERT( mdfa1 == mdfa2);
}

void re2faTest::testBrzozowski() {
	std::string input = "a+a* b*";
	std::stringstream inputs(input);

	regexp::RegExpFromStringParser parser(inputs);
	regexp::RegExp regexp1( parser.parseValue() );

	re2fa::Brzozowski brzozowski1;
	automaton::DFA dfa1 = brzozowski1.convert(regexp1);

	regexp::RegExp regexp2( fa2re::BrzozowskiAlgebraic::convert(dfa1) );

	re2fa::Brzozowski brzozowski2;
	automaton::DFA dfa2 = brzozowski2.convert(regexp2);

	automaton::DFA mdfa1 = minimize::MinimizeDFA::minimize(dfa1);
	automaton::DFA mdfa2 = minimize::MinimizeDFA::minimize(dfa2);

	CPPUNIT_ASSERT( mdfa1 == mdfa2);
}
