/*
 * FromXMLParsers.hpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef FROM_XML_PARSERS_HPP_
#define FROM_XML_PARSERS_HPP_

#include "label/LabelFromXMLParser.h"
#include "alphabet/SymbolFromXMLParser.h"
#include "regexp/RegExpFromXMLParser.h"
#include "string/StringFromXMLParser.h"
#include "automaton/AutomatonFromXMLParser.h"
#include "grammar/GrammarFromXMLParser.h"
#include "object/ObjectFromXMLParser.h"
#include "exception/ExceptionFromXMLParser.h"

#include "label/LabelToXMLComposer.h"
#include "alphabet/SymbolToXMLComposer.h"
#include "regexp/RegExpToXMLComposer.h"
#include "string/StringToXMLComposer.h"
#include "automaton/AutomatonToXMLComposer.h"
#include "grammar/GrammarToXMLComposer.h"
#include "object/ObjectToXMLComposer.h"
#include "exception/ExceptionToXMLComposer.h"

#include "object/ObjectBase.h"

namespace alib {

template<typename T>
struct api {
	static T parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const T& data);
};

template<typename T>
struct api<std::set<T>> {
	static std::set<T> parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const std::set<T>& data);
};

template<>
struct api<container::ObjectsSet> {
	static container::ObjectsSet parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const container::ObjectsSet& data);
};


template<>
struct api<Object> {
	static Object parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const Object& data);
};


template<>
struct api<exception::AlibException> {
	static exception::AlibException parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const exception::AlibException& data);
};


template<>
struct api<alphabet::Symbol> {
	static alphabet::Symbol parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const alphabet::Symbol& data);
};

template<>
struct api<alphabet::LabeledSymbol> {
	static alphabet::LabeledSymbol parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const alphabet::LabeledSymbol& data);
};

template<>
struct api<alphabet::BlankSymbol> {
	static alphabet::BlankSymbol parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const alphabet::BlankSymbol& data);
};

template<>
struct api<alphabet::BottomOfTheStackSymbol> {
	static alphabet::BottomOfTheStackSymbol parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const alphabet::BottomOfTheStackSymbol& data);
};

template<>
struct api<alphabet::EndSymbol> {
	static alphabet::EndSymbol parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const alphabet::EndSymbol& data);
};


template<>
struct api<automaton::Automaton> {
	static automaton::Automaton parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::Automaton& data);
};

template<>
struct api<automaton::UnknownAutomaton> {
	static automaton::UnknownAutomaton parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::UnknownAutomaton& data);
};

template<>
struct api<automaton::EpsilonNFA> {
	static automaton::EpsilonNFA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::EpsilonNFA& data);
};

template<>
struct api<automaton::NFA> {
	static automaton::NFA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::NFA& data);
};

template<>
struct api<automaton::DFA> {
	static automaton::DFA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::DFA& data);
};

template<>
struct api<automaton::ExtendedNFA> {
	static automaton::ExtendedNFA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::ExtendedNFA& data);
};

template<>
struct api<automaton::CompactNFA> {
	static automaton::CompactNFA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::CompactNFA& data);
};

template<>
struct api<automaton::DPDA> {
	static automaton::DPDA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::DPDA& data);
};

template<>
struct api<automaton::SinglePopDPDA> {
	static automaton::SinglePopDPDA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::SinglePopDPDA& data);
};

template<>
struct api<automaton::NPDA> {
	static automaton::NPDA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::NPDA& data);
};

template<>
struct api<automaton::SinglePopNPDA> {
	static automaton::SinglePopNPDA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::SinglePopNPDA& data);
};

template<>
struct api<automaton::OneTapeDTM> {
	static automaton::OneTapeDTM parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::OneTapeDTM& data);
};


template<>
struct api<grammar::Grammar> {
	static grammar::Grammar parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::Grammar& data);
};

template<>
struct api<grammar::UnknownGrammar> {
	static grammar::UnknownGrammar parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::UnknownGrammar& data);
};

template<>
struct api<grammar::LeftLG> {
	static grammar::LeftLG parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::LeftLG& data);
};

template<>
struct api<grammar::LeftRG> {
	static grammar::LeftRG parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::LeftRG& data);
};

template<>
struct api<grammar::RightLG> {
	static grammar::RightLG parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::RightLG& data);
};

template<>
struct api<grammar::RightRG> {
	static grammar::RightRG parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::RightRG& data);
};

template<>
struct api<grammar::LG> {
	static grammar::LG parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::LG& data);
};

template<>
struct api<grammar::CFG> {
	static grammar::CFG parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::CFG& data);
};

template<>
struct api<grammar::EpsilonFreeCFG> {
	static grammar::EpsilonFreeCFG parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::EpsilonFreeCFG& data);
};

template<>
struct api<grammar::CNF> {
	static grammar::CNF parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::CNF& data);
};

template<>
struct api<grammar::GNF> {
	static grammar::GNF parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::GNF& data);
};

template<>
struct api<grammar::CSG> {
	static grammar::CSG parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::CSG& data);
};

template<>
struct api<grammar::NonContractingGrammar> {
	static grammar::NonContractingGrammar parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::NonContractingGrammar& data);
};

template<>
struct api<grammar::ContextPreservingUnrestrictedGrammar> {
	static grammar::ContextPreservingUnrestrictedGrammar parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::ContextPreservingUnrestrictedGrammar& data);
};

template<>
struct api<grammar::UnrestrictedGrammar> {
	static grammar::UnrestrictedGrammar parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::UnrestrictedGrammar& data);
};


template<>
struct api<label::Label> {
	static label::Label parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const label::Label& data);
};

template<>
struct api<label::StringLabel> {
	static label::StringLabel parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const label::StringLabel& data);
};

template<>
struct api<label::IntegerLabel> {
	static label::IntegerLabel parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const label::IntegerLabel& data);
};

template<>
struct api<label::CharacterLabel> {
	static label::CharacterLabel parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const label::CharacterLabel& data);
};

template<>
struct api<label::LabelSetLabel> {
	static label::LabelSetLabel parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const label::LabelSetLabel& data);
};

template<>
struct api<label::LabelPairLabel> {
	static label::LabelPairLabel parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const label::LabelPairLabel& data);
};


template<>
struct api<regexp::RegExp> {
	static regexp::RegExp parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const regexp::RegExp& data);
};

template<>
struct api<regexp::UnboundedRegExp> {
	static regexp::UnboundedRegExp parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const regexp::UnboundedRegExp& data);
};

template<>
struct api<regexp::FormalRegExp> {
	static regexp::FormalRegExp parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const regexp::FormalRegExp& data);
};


template<>
struct api<string::String> {
	static string::String parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const string::String& data);
};

template<>
struct api<string::LinearString> {
	static string::LinearString parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const string::LinearString& data);
};

template<>
struct api<string::CyclicString> {
	static string::CyclicString parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const string::CyclicString& data);
};

template<>
struct api<string::Epsilon> {
	static string::Epsilon parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const string::Epsilon& data);
};

} /* namespace alib */

#include "container/ContainerFromXMLParser.hpp"

#include "container/ContainerToXMLComposer.hpp"

namespace alib {

template<>
struct api<container::Container> {
	static container::Container parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const container::Container& data);
};

/**
 * Basic exception from which are derived all other exceptions.
 * Contains reason why the exception occured.
 */
class FromXMLParsers {
public:
	static const label::LabelFromXMLParser labelParser;
	static const alphabet::SymbolFromXMLParser symbolParser;
	static const regexp::RegExpFromXMLParser regexpParser;
	static const string::StringFromXMLParser stringParser;
	static const automaton::AutomatonFromXMLParser automatonParser;
	static const grammar::GrammarFromXMLParser grammarParser;
	static const exception::ExceptionFromXMLParser exceptionParser;
	static const ObjectFromXMLParser objectParser;
	static const container::ContainerFromXMLParser containerParser;

};

class ToXMLComposers : public VisitableObjectBase::const_visitor_type {
	void Visit(void*, const exception::AlibException& symbol) const;

	void Visit(void*, const alphabet::LabeledSymbol& symbol) const;
	void Visit(void*, const alphabet::BlankSymbol& symbol) const;
	void Visit(void*, const alphabet::BottomOfTheStackSymbol& symbol) const;
	void Visit(void*, const alphabet::EndSymbol& symbol) const;

	void Visit(void*, const automaton::UnknownAutomaton& automaton) const;
	void Visit(void*, const automaton::EpsilonNFA& automaton) const;
	void Visit(void*, const automaton::NFA& automaton) const;
	void Visit(void*, const automaton::DFA& automaton) const;
	void Visit(void*, const automaton::ExtendedNFA& automaton) const;
	void Visit(void*, const automaton::CompactNFA& automaton) const;
	void Visit(void*, const automaton::DPDA& automaton) const;
	void Visit(void*, const automaton::SinglePopDPDA& automaton) const;
	void Visit(void*, const automaton::NPDA& automaton) const;
	void Visit(void*, const automaton::SinglePopNPDA& automaton) const;
	void Visit(void*, const automaton::OneTapeDTM& automaton) const;

	void Visit(void*, const grammar::UnknownGrammar& grammar) const;
	void Visit(void*, const grammar::LeftLG& grammar) const;
	void Visit(void*, const grammar::LeftRG& grammar) const;
	void Visit(void*, const grammar::RightLG& grammar) const;
	void Visit(void*, const grammar::RightRG& grammar) const;
	void Visit(void*, const grammar::LG& grammar) const;
	void Visit(void*, const grammar::CFG& grammar) const;
	void Visit(void*, const grammar::EpsilonFreeCFG& grammar) const;
	void Visit(void*, const grammar::CNF& grammar) const;
	void Visit(void*, const grammar::GNF& grammar) const;
	void Visit(void*, const grammar::CSG& grammar) const;
	void Visit(void*, const grammar::NonContractingGrammar& grammar) const;
	void Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar& grammar) const;
	void Visit(void*, const grammar::UnrestrictedGrammar& grammar) const;

	void Visit(void*, const label::StringLabel& label) const;
	void Visit(void*, const label::IntegerLabel& label) const;
	void Visit(void*, const label::CharacterLabel& label) const;
	void Visit(void*, const label::LabelSetLabel& label) const;
	void Visit(void*, const label::LabelPairLabel& label) const;

	void Visit(void*, const regexp::UnboundedRegExp& regexp) const;
	void Visit(void*, const regexp::FormalRegExp& regexp) const;

	void Visit(void*, const string::LinearString& string) const;
	void Visit(void*, const string::CyclicString& string) const;
	void Visit(void*, const string::Epsilon& string) const;

	void Visit(void*, const container::ObjectsSet& set) const;

public:
	static const label::LabelToXMLComposer labelComposer;
	static const alphabet::SymbolToXMLComposer symbolComposer;
	static const regexp::RegExpToXMLComposer regexpComposer;
	static const string::StringToXMLComposer stringComposer;
	static const automaton::AutomatonToXMLComposer automatonComposer;
	static const grammar::GrammarToXMLComposer grammarComposer;
	static const exception::ExceptionToXMLComposer exceptionComposer;
	static const ObjectToXMLComposer objectComposer;
	static const container::ContainerToXMLComposer containerComposer;

	static const ToXMLComposers toXMLComposers;
};

template<typename T>
std::set<T> api<std::set<T>>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::containerParser.parseSet<T>(input);
}

template<typename T>
std::list<sax::Token> api<std::set<T>>::compose(const std::set<T>& input) {
	return ToXMLComposers::containerComposer.compose<T>(input);
}

} /* namespace alib */

#endif /* FROM_XML_PARSERS_HPP_ */
