/*
 * FiniteAutomatonFromStringParser.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef FINITE_AUTOMATON_FROM_STRING_PARSER_H_
#define FINITE_AUTOMATON_FROM_STRING_PARSER_H_

#include "FiniteAutomatonFromStringLexer.h"
#include "../../alphabet/SymbolFromStringParser.h"
#include "../../label/LabelFromStringParser.h"
#include "../../FromStringParser.hpp"

#include <tuple>

#include "../Automaton.h"
#include "../AutomatonFeatures.h"

#include "../common/State.h"

#include "../../std/variant.hpp"

#include "../../string/Epsilon.h"

namespace automaton {

class FiniteAutomatonFromStringParser : public alib::FromStringParser<Automaton, FEATURES> {
	FiniteAutomatonFromStringLexer m_FiniteAutomatonLexer;
	alphabet::SymbolFromStringParser m_SymbolParser;
	label::LabelFromStringParser m_LabelParser;

	void initialFinalState(bool& initial, bool& final);

	EpsilonNFA parseEpsilonNFA();
	NFA parseNFA();
	DFA parseDFA();
	void parseEpsilonNFATransition(EpsilonNFA& res, const std::vector<std::variant<string::Epsilon, alphabet::Symbol> >& symbols);
	void parseNFATransition(NFA& res, const std::vector<alphabet::Symbol>& symbols);
	void parseDFATransition(std::set<State>& states, const std::vector<alphabet::Symbol>& symbols, State*& initialState, std::set<State>& finalStates, std::set<std::tuple<State, alphabet::Symbol, State>>& transitionFunction);
	virtual bool last();
	Automaton parse();
	Automaton parse(const std::set<FEATURES>& features);
	bool next();

public:
	FiniteAutomatonFromStringParser(std::istream& input);

	bool first();

};

} /* namespace automaton */

#endif /* FINITE_AUTOMATON_FROM_STRING_PARSER_H_ */
