/*
 * MultiInitialSymbols.cpp
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#include "MultiInitialSymbolsPushdownStoreAlphabet.h"
#include "../AutomatonException.h"
#include <algorithm>

namespace automaton {

bool MultiInitialSymbolsPushdownStoreAlphabet::addInitialSymbol(const alphabet::Symbol& start) {
	if (stackAlphabet.find(start) == stackAlphabet.end()) {
		throw AutomatonException("Stack symbol \"" + (std::string) start + "\" doesn't exist.");
	}
	
	return this->initialSymbols.insert(start).second;
}

bool MultiInitialSymbolsPushdownStoreAlphabet::removeInitialSymbol(const alphabet::Symbol& start) {
	return this->initialSymbols.erase(start);
}

void MultiInitialSymbolsPushdownStoreAlphabet::setInitialSymbols(const std::set<alphabet::Symbol>& symbols) {
	std::set<alphabet::Symbol> tmp;
	std::set_difference(symbols.begin(), symbols.end(), this->stackAlphabet.begin(), this->stackAlphabet.end(), std::inserter(tmp, tmp.end()));
	if(tmp.size() != 0)
		throw AutomatonException("Initial symbols not in stack alphabet");
	
	this->initialSymbols = symbols;
}

const std::set<alphabet::Symbol>& MultiInitialSymbolsPushdownStoreAlphabet::getInitialSymbols() const {
	return initialSymbols;
}

} /* namespace automaton */
