/*
 * ContainerFromXMLParser.h
 *
 *  Created on: Oct 12, 2013
 *      Author: Jan Travnicek
 */

#ifndef CONTAINER_FROM_XML_PARSER_H_
#define CONTAINER_FROM_XML_PARSER_H_

#include "../sax/FromXMLParserHelper.h"
#include "ContainerFeatures.h"

#include <set>
#include "ObjectsSet.h"

#include "Container.h"

namespace alib {

template<typename T>
struct api;

} /* namespace alib */

namespace container {

/**
 * Parser used to get general FSM or EpsilonNFA, NFA, DFA from XML parsed into list of Tokens.
 */
class ContainerFromXMLParser : public sax::FromXMLParserHelper {
	Container parseContainer(std::list<sax::Token>& input) const;
	Container parseContainer(std::list<sax::Token>& input, const std::set<FEATURES>& features) const;

	template<typename T>
	std::set<T> parseSet(std::list<sax::Token>& input) const;

	ObjectsSet parseObjectsSet(std::list<sax::Token>& input) const;

	template<typename T> friend class alib::api;

public:
	bool first(std::list<sax::Token>& input) const;
};

} /* namespace container */

#include "../Api.hpp"

namespace container {

template<typename T>
std::set<T> ContainerFromXMLParser::parseSet(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "set");

	std::set<T> set;

	while(isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		set.insert(alib::api<T>::parse(input));
	}

	popToken(input, sax::Token::TokenType::END_ELEMENT, "set");
	return set;
}

} /* namespace container */

#endif /* CONTAINER_FROM_XML_PARSER_H_ */

