#include "AutomatonRevertor.h"
#include "../exception/AlibException.h"

#include "../automaton/UnknownAutomaton.h"
#include "../automaton/FSM/DFA.h"
#include "../automaton/FSM/NFA.h"
#include "../automaton/FSM/EpsilonNFA.h"
#include "../automaton/FSM/ExtendedNFA.h"
#include "../automaton/FSM/CompactNFA.h"
#include "../automaton/PDA/NPDA.h"
#include "../automaton/PDA/SinglePopNPDA.h"
#include "../automaton/TM/OneTapeDTM.h"

namespace automaton {

UnknownAutomaton AutomatonRevertor::revert(const UnknownAutomaton& automaton) const {
	return automaton;
}

UnknownAutomaton AutomatonRevertor::revert(const DFA& automaton) const {
	UnknownAutomaton out;
	// TODO
	return out;
}

UnknownAutomaton AutomatonRevertor::revert(const NFA& automaton) const {
	UnknownAutomaton out;
	// TODO
	return out;
}

UnknownAutomaton AutomatonRevertor::revert(const EpsilonNFA& automaton) const {
	UnknownAutomaton out;
	// TODO
	return out;
}

UnknownAutomaton AutomatonRevertor::revert(const ExtendedNFA&) const {
	throw exception::AlibException("Unable to revert ExtendedNFA to UnknownAutomaton");
}

UnknownAutomaton AutomatonRevertor::revert(const CompactNFA&) const {
	throw exception::AlibException("Unable to revert CompactNFA to UnknownAutomaton");
}

UnknownAutomaton AutomatonRevertor::revert(const NPDA& automaton) const {
	UnknownAutomaton out;
	// TODO
	return out;
}

UnknownAutomaton AutomatonRevertor::revert(const SinglePopNPDA& automaton) const {
	UnknownAutomaton out;
	// TODO
	return out;
}

UnknownAutomaton AutomatonRevertor::revert(const OneTapeDTM& automaton) const {
	UnknownAutomaton out;
	// TODO
	return out;
}

UnknownAutomaton AutomatonRevertor::revert(const Automaton& automaton) const {
	UnknownAutomaton out;
	automaton.getData().Accept((void*) &out, *this);
	return out;
}

void AutomatonRevertor::Visit(void* data, const UnknownAutomaton& automaton) const {
	*((UnknownAutomaton*) data) = this->revert(automaton);
}

void AutomatonRevertor::Visit(void* data, const EpsilonNFA& automaton) const {
	*((UnknownAutomaton*) data) = this->revert(automaton);
}

void AutomatonRevertor::Visit(void* data, const NFA& automaton) const {
	*((UnknownAutomaton*) data) = this->revert(automaton);
}

void AutomatonRevertor::Visit(void* data, const DFA& automaton) const {
	*((UnknownAutomaton*) data) = this->revert(automaton);
}

void AutomatonRevertor::Visit(void* data, const ExtendedNFA& automaton) const {
	*((UnknownAutomaton*) data) = this->revert(automaton);
}

void AutomatonRevertor::Visit(void* data, const CompactNFA& automaton) const {
	*((UnknownAutomaton*) data) = this->revert(automaton);
}

void AutomatonRevertor::Visit(void* data, const NPDA& automaton) const {
	*((UnknownAutomaton*) data) = this->revert(automaton);
}

void AutomatonRevertor::Visit(void* data, const SinglePopNPDA& automaton) const {
	*((UnknownAutomaton*) data) = this->revert(automaton);
}

void AutomatonRevertor::Visit(void* data, const OneTapeDTM& automaton) const {
	*((UnknownAutomaton*) data) = this->revert(automaton);
}

} /* namespace automaton */
