/*
 * GNF.h
 *
 *  Created on: Nov 17, 2013
 *      Author: Jan Travnicek
 */

#ifndef GNF_H_
#define GNF_H_

#include "../GrammarBase.h"
#include <map>
#include <vector>
#include <sstream>
#include "../common/TerminalNonterminalAlphabetInitialSymbol.h"

namespace grammar {

/**
 * Context free grammar in greibach normal form. Type 2 in Chomsky hierarchy. Produces context free languages.
 */
class GNF : public std::acceptor<GNF, VisitableGrammarBase, std::acceptor<GNF, alib::VisitableObjectBase, GrammarBase> >, public TerminalNonterminalAlphabetInitialSymbol {
	std::map<alphabet::Symbol, std::set<std::pair<alphabet::Symbol, std::vector<alphabet::Symbol> >> > rules;
	bool generatesEpsilon;
public:
	GNF(const alphabet::Symbol& initialSymbol);

	GNF(const std::set<alphabet::Symbol>& nonTerminalSymbols, const std::set<alphabet::Symbol>& terminalSymbols, const alphabet::Symbol& initialSymbol);

	virtual GrammarBase* clone() const;

	virtual GrammarBase* plunder() &&;

	bool addRule(const alphabet::Symbol& leftHandSide, const std::pair<alphabet::Symbol, std::vector<alphabet::Symbol> >& rightHandSide);

	const std::map<alphabet::Symbol, std::set<std::pair<alphabet::Symbol, std::vector<alphabet::Symbol> >> > & getRules() const;

	bool removeRule(const alphabet::Symbol& leftHandSide, const std::pair<alphabet::Symbol, std::vector<alphabet::Symbol> >& rightHandSide);

	bool addRawRule(const alphabet::Symbol& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide);

	std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>> getRawRules() const;

	bool removeRawRule(const alphabet::Symbol& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide);

	bool removeTerminalSymbol(const alphabet::Symbol& symbol);

	bool removeNonterminalSymbol(const alphabet::Symbol& symbol);

	void setGeneratesEpsilon(bool genEps);
	bool getGeneratesEpsilon() const;

	virtual bool operator <(const alib::ObjectBase& other) const;
	virtual bool operator ==(const alib::ObjectBase& other) const;
	virtual bool operator >(const alib::ObjectBase& other) const;

	virtual bool operator==(const GNF& other) const;

	virtual bool operator<(const GNF& other) const;

	virtual void operator>>(std::ostream& os) const;

	virtual operator std::string () const;

	virtual int selfTypeId() const {
		return typeId(*this);
	}
};

} /* namespace grammar */

#endif /* GNF_H_ */
