#include "GrammarToStringComposer.h"
#include "../alphabet/SymbolToStringComposer.h"
#include <sstream>

namespace grammar {

std::string GrammarToStringComposer::compose(const GrammarBase& grammar) const {

}

std::string GrammarToStringComposer::compose(const Grammar& grammar) const {
	std::stringstream out;
	grammar.getData().Accept((void*) &out, *this);
	return out.str();
}

std::string GrammarToStringComposer::compose( const UnknownGrammar & grammar ) const {
	alphabet::SymbolToStringComposer composer;
	std::stringstream ss;

	// group by left side
	std::map< std::vector<alphabet::Symbol>, std::set<std::vector<alphabet::Symbol> > > group;
	for( const auto & r : grammar.getRules( ) ) {
		group[ r.getLeftHandSide( ) ].insert( r.getRightHandSide() );
	}

	ss << "(";

	auto last = grammar.getNonterminalAlphabet( ).end( );
	last--;

	ss << "{";
	for( auto it = grammar.getNonterminalAlphabet( ).begin( ) ; it != grammar.getNonterminalAlphabet( ).end( ) ; it++ )
		ss << composer.compose(*it) << ( it == last ? "" : ", " );
	ss << "}," << std::endl << " ";

	last = grammar.getTerminalAlphabet( ).end( );
	last--;

	ss << "{";
	for( auto it = grammar.getTerminalAlphabet( ).begin( ) ; it != grammar.getTerminalAlphabet( ).end( ) ; it++ )
		ss << composer.compose(*it) << ( it == last ? "" : ", " );
	ss << "}," << std::endl << " ";

	ss << "{";
	bool first = true;
	for( const auto & g : group ) {
		if(first)
			first = false;
		else
			ss << ",\n  ";

		for( const auto & symbol : g.first )
			ss << composer.compose( symbol ) << " ";
		
		ss << "->";

		bool ffirst = true;
		for( const auto & rhs : g.second ) {
			if(ffirst)
				ffirst = false;
			else
				ss << " |";

			if(rhs.size())
				for( const auto & symbol : rhs )
					ss << " " << composer.compose( symbol );
			else
				ss << " #E";
		}
	}
	ss << "}," << std::endl << " ";

	ss << composer.compose(grammar.getInitialSymbol( ));
	ss << ")" << std::endl;
	return std::move(ss).str();
}

std::string GrammarToStringComposer::compose(const LeftLG& grammar) const {
	// TODO
}

std::string GrammarToStringComposer::compose(const LeftRG& grammar) const {
	// TODO
}

std::string GrammarToStringComposer::compose(const RightLG& grammar) const {
	// TODO
}

std::string GrammarToStringComposer::compose(const RightRG& grammar) const {
	// TODO
}

std::string GrammarToStringComposer::compose(const LG& grammar) const {
	// TODO
}

std::string GrammarToStringComposer::compose(const CFG& grammar) const {
	// TODO
}

std::string GrammarToStringComposer::compose(const EpsilonFreeCFG& grammar) const {
	// TODO
}

std::string GrammarToStringComposer::compose(const CNF& grammar) const {
	// TODO
}

std::string GrammarToStringComposer::compose(const GNF& grammar) const {
	// TODO
}

std::string GrammarToStringComposer::compose(const CSG& grammar) const {
	// TODO
}

std::string GrammarToStringComposer::compose(const NonContractingGrammar& grammar) const {
	// TODO
}

std::string GrammarToStringComposer::compose(const ContextPreservingUnrestrictedGrammar& grammar) const {
	// TODO
}

std::string GrammarToStringComposer::compose(const UnrestrictedGrammar& grammar) const {
	// TODO
}

void GrammarToStringComposer::Visit(void* data, const UnknownGrammar& grammar) const {
	*((std::stringstream*) data) << this->compose(grammar);
}

void GrammarToStringComposer::Visit(void* data, const LeftLG& grammar) const {
	*((std::stringstream*) data) << this->compose(grammar);
}

void GrammarToStringComposer::Visit(void* data, const LeftRG& grammar) const {
	*((std::stringstream*) data) << this->compose(grammar);
}

void GrammarToStringComposer::Visit(void* data, const RightLG& grammar) const {
	*((std::stringstream*) data) << this->compose(grammar);
}

void GrammarToStringComposer::Visit(void* data, const RightRG& grammar) const {
	*((std::stringstream*) data) << this->compose(grammar);
}

void GrammarToStringComposer::Visit(void* data, const LG& grammar) const {
	*((std::stringstream*) data) << this->compose(grammar);
}

void GrammarToStringComposer::Visit(void* data, const CFG& grammar) const {
	*((std::stringstream*) data) << this->compose(grammar);
}

void GrammarToStringComposer::Visit(void* data, const EpsilonFreeCFG& grammar) const {
	*((std::stringstream*) data) << this->compose(grammar);
}

void GrammarToStringComposer::Visit(void* data, const CNF& grammar) const {
	*((std::stringstream*) data) << this->compose(grammar);
}

void GrammarToStringComposer::Visit(void* data, const GNF& grammar) const {
	*((std::stringstream*) data) << this->compose(grammar);
}

void GrammarToStringComposer::Visit(void* data, const CSG& grammar) const {
	*((std::stringstream*) data) << this->compose(grammar);
}

void GrammarToStringComposer::Visit(void* data, const NonContractingGrammar& grammar) const {
	*((std::stringstream*) data) << this->compose(grammar);
}

void GrammarToStringComposer::Visit(void* data, const ContextPreservingUnrestrictedGrammar& grammar) const {
	*((std::stringstream*) data) << this->compose(grammar);
}

void GrammarToStringComposer::Visit(void* data, const UnrestrictedGrammar& grammar) const {
	*((std::stringstream*) data) << this->compose(grammar);
}

}
