/*
 * LabelSetLabel.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "LabelSetLabel.h"
#include "../std/set.hpp"
#include <sstream>

namespace label {

LabelSetLabel::LabelSetLabel(std::set<Label> label) : label(label) {

}

LabelBase* LabelSetLabel::clone() const {
	return new LabelSetLabel(*this);
}

LabelBase* LabelSetLabel::plunder() && {
	return new LabelSetLabel(std::move(*this));
}

std::set<Label> LabelSetLabel::getData() const {
	return label;
}

bool LabelSetLabel::operator<(const alib::ObjectBase& other) const {
	return other > *this;
}

bool LabelSetLabel::operator==(const alib::ObjectBase& other) const {
	return other == *this;
}

bool LabelSetLabel::operator >(const alib::ObjectBase& other) const {
	return other < *this;
}


bool LabelSetLabel::operator <(const LabelSetLabel& other) const {
	return label < other.label;
}

bool LabelSetLabel::operator ==(const LabelSetLabel& other) const {
	return label == other.label;
}

void LabelSetLabel::operator>>(std::ostream& out) const {
	out << "(LabelSetLabel " << label << ")";
}

LabelSetLabel::operator std::string() const {
	std::stringstream ss;
	ss << "{";
	bool first = true;
	for(const Label& subLabel : label) {
		if(!first) 
			ss << ", ";
		else
			first = false;
		ss << (std::string) subLabel;
	}
	ss << "}";
	return std::move(ss).str();
}

} /* namespace label */

