/*
 * LabelToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "LabelToStringComposer.h"
#include <algorithm>
#include "IntegerLabel.h"
#include "StringLabel.h"
#include "CharacterLabel.h"
#include "LabelSetLabel.h"
#include "LabelPairLabel.h"

namespace label {

void LabelToStringComposer::Visit(void* userData, const label::StringLabel& label) {
	std::stringstream &out = *((std::stringstream*) userData);

	auto replace = [](std::string& str, const std::string& what, const std::string& with) {
		size_t index = 0;
		while((index = str.find(what, index)) != std::string::npos) {
			str.replace(index, what.length(), with);
			index += with.length();
		}
	};
	
	std::string tmp = label.getData();
	replace(tmp, "'", "\\'" );
	out << '\'' << tmp << '\'';
}

void LabelToStringComposer::Visit(void* userData, const CharacterLabel& label) {
	std::stringstream &out = *((std::stringstream*) userData);

	out << label.getData();
}

void LabelToStringComposer::Visit(void* userData, const IntegerLabel& label) {
	std::stringstream &out = *((std::stringstream*) userData);

	out << label.getData();
}

void LabelToStringComposer::Visit(void* userData, const LabelSetLabel& label) {
	std::stringstream &out = *((std::stringstream*) userData);

	out << '[';
	bool first = true;
	for(const Label& innerLabel : label.getData()) {
		if(!first)
			out << ", ";
		else
			first = false;
		innerLabel.getData().Accept(userData, *this);
	}
	out << ']';
}

void LabelToStringComposer::Visit(void* userData, const LabelPairLabel& label) {
	std::stringstream &out = *((std::stringstream*) userData);

	out << '<';
	label.getData().first.getData().Accept(userData, *this);
	out << ", ";
	label.getData().second.getData().Accept(userData, *this);
	out << '>';
}

std::string LabelToStringComposer::compose(const Label& label) {
	std::stringstream out;
	label.getData().Accept((void*) &out, *this);
	return std::move(out).str();
}

} /* namespace label */
