/*
 * LabelToXMLComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "LabelToXMLComposer.h"
#include "IntegerLabel.h"
#include "StringLabel.h"
#include "CharacterLabel.h"
#include "../std/itos.h"

#include "../Api.hpp"

namespace label {

std::list<sax::Token> LabelToXMLComposer::compose(const Label& label) const {
	std::list<sax::Token> out;
	label.getData().Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
	return out;
}

std::list<sax::Token> LabelToXMLComposer::compose(const LabelBase& label) const {
	std::list<sax::Token> out;
	label.Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
	return out;
}

std::list<sax::Token> LabelToXMLComposer::compose(const IntegerLabel& label) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("IntegerLabel", sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token(std::itos(label.getData()), sax::Token::TokenType::CHARACTER));
	out.push_back(sax::Token("IntegerLabel", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> LabelToXMLComposer::compose(const CharacterLabel& label) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("CharacterLabel", sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token(std::string(1, label.getData()), sax::Token::TokenType::CHARACTER));
	out.push_back(sax::Token("CharacterLabel", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> LabelToXMLComposer::compose(const StringLabel& label) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("StringLabel", sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token(label.getData(), sax::Token::TokenType::CHARACTER));
	out.push_back(sax::Token("StringLabel", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> LabelToXMLComposer::compose(const LabelSetLabel& label) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("LabelSetLabel", sax::Token::TokenType::START_ELEMENT));
	for(const Label& innerLabel : label.getData()) {
		out.splice(out.end(), alib::api<label::Label>::compose(innerLabel));
	}
	out.push_back(sax::Token("LabelSetLabel", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> LabelToXMLComposer::compose(const LabelPairLabel& label) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("LabelPairLabel", sax::Token::TokenType::START_ELEMENT));
	out.splice(out.end(), alib::api<label::Label>::compose(label.getData().first));
	out.splice(out.end(), alib::api<label::Label>::compose(label.getData().second));
	out.push_back(sax::Token("LabelPairLabel", sax::Token::TokenType::END_ELEMENT));
	return out;
}

} /* namespace label */
