/*
 * StringLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef STRING_LABEL_H_
#define STRING_LABEL_H_

#include <string>
#include <ostream>

#include "LabelBase.h"

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class StringLabel : public std::acceptor<StringLabel, VisitableLabelBase, std::acceptor<StringLabel, alib::VisitableObjectBase, LabelBase> > {
protected:
	std::string label;
public:
	virtual LabelBase* clone() const;

	virtual LabelBase* plunder() &&;

	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit StringLabel(const std::string& label);
	explicit StringLabel(std::string&& label);

	virtual bool operator<(const alib::ObjectBase& other) const;
	
	virtual bool operator>(const alib::ObjectBase& other) const;

	virtual bool operator==(const alib::ObjectBase& other) const;

	/**
	 * @return name of the symbol
	 */
	const std::string& getData() const;

	virtual bool operator <(const StringLabel& other) const;
	virtual bool operator ==(const StringLabel& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual operator std::string () const;

	virtual int selfTypeId() const {
		return typeId(*this);
	}
};

} /* namespace label */

#endif /* STRING_LABEL_H_ */

