/*
 * ObjectBase.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef OBJECT_BASE_H_
#define OBJECT_BASE_H_

#include "../std/visitor.hpp"
#include "../common/base.hpp"

namespace exception {

class AlibException;

}

namespace automaton {

class UnknownAutomaton;
class DFA;
class NFA;
class EpsilonNFA;
class CompactNFA;
class ExtendedNFA;
class DPDA;
class SinglePopDPDA;
class NPDA;
class SinglePopNPDA;
class OneTapeDTM;

}

namespace grammar {

class UnknownGrammar;
class LeftLG;
class LeftRG;
class RightLG;
class RightRG;
class LG;
class CFG;
class EpsilonFreeCFG;
class CNF;
class GNF;
class CSG;
class NonContractingGrammar;
class ContextPreservingUnrestrictedGrammar;
class UnrestrictedGrammar;

}

namespace label {

class StringLabel;
class IntegerLabel;
class CharacterLabel;
class LabelSetLabel;
class LabelPairLabel;

}

namespace regexp {

class UnboundedRegExp;
class FormalRegExp;

}

namespace string {

class LinearString;
class CyclicString;
class Epsilon;

}

namespace alphabet {

class LabeledSymbol;
class BlankSymbol;
class BottomOfTheStackSymbol;
class EndSymbol;

}

namespace container {

class ObjectsSet;

}

namespace alib {

typedef std::acceptor_base<
			exception::AlibException,
			automaton::UnknownAutomaton, automaton::DFA, automaton::NFA, automaton::EpsilonNFA, automaton::CompactNFA, automaton::ExtendedNFA, automaton::DPDA, automaton::SinglePopDPDA, automaton::NPDA, automaton::SinglePopNPDA, automaton::OneTapeDTM,
			grammar::UnknownGrammar, grammar::LeftLG, grammar::LeftRG, grammar::RightLG, grammar::RightRG, grammar::LG, grammar::CFG, grammar::EpsilonFreeCFG, grammar::CNF, grammar::GNF, grammar::CSG, grammar::NonContractingGrammar, grammar::ContextPreservingUnrestrictedGrammar, grammar::UnrestrictedGrammar,
			label::StringLabel, label::IntegerLabel, label::CharacterLabel, label::LabelSetLabel, label::LabelPairLabel,
			regexp::UnboundedRegExp, regexp::FormalRegExp,
			string::Epsilon, string::LinearString, string::CyclicString,
			alphabet::LabeledSymbol, alphabet::BlankSymbol, alphabet::BottomOfTheStackSymbol, alphabet::EndSymbol,
			container::ObjectsSet
	> VisitableObjectBase;

class ObjectBase :
	public alib::base<
			ObjectBase,
			exception::AlibException,
			automaton::UnknownAutomaton, automaton::DFA, automaton::NFA, automaton::EpsilonNFA, automaton::CompactNFA, automaton::ExtendedNFA, automaton::DPDA, automaton::SinglePopDPDA, automaton::NPDA, automaton::SinglePopNPDA, automaton::OneTapeDTM,
			grammar::UnknownGrammar, grammar::LeftLG, grammar::LeftRG, grammar::RightLG, grammar::RightRG, grammar::LG, grammar::CFG, grammar::EpsilonFreeCFG, grammar::CNF, grammar::GNF, grammar::CSG, grammar::NonContractingGrammar, grammar::ContextPreservingUnrestrictedGrammar, grammar::UnrestrictedGrammar,
			label::StringLabel, label::IntegerLabel, label::CharacterLabel, label::LabelSetLabel, label::LabelPairLabel,
			regexp::UnboundedRegExp, regexp::FormalRegExp,
			string::Epsilon, string::LinearString, string::CyclicString,
			alphabet::LabeledSymbol, alphabet::BlankSymbol, alphabet::BottomOfTheStackSymbol, alphabet::EndSymbol,
			container::ObjectsSet
	>, public VisitableObjectBase {
};

} /* namespace alib */

#endif /* OBJECT_BASE_H_ */
