/*
 * ObjectFromXMLParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "ObjectFromXMLParser.h"

#include "../sax/ParserException.h"

#include "../Api.hpp"
#include <iostream>

namespace alib {

Object ObjectFromXMLParser::parseObject(std::list<sax::Token>& input) const {
	ObjectBase* tmp;

	if(alib::FromXMLParsers::symbolParser.first(input)) {
		alphabet::Symbol symbol = alib::api<alphabet::Symbol>::parse(input);
		tmp = std::move(symbol.getData()).plunder();
	} else if(alib::FromXMLParsers::exceptionParser.first(input)) {
		exception::AlibException exception = alib::api<exception::AlibException>::parse(input);
		tmp = std::move(exception).plunder();
	} else if(alib::FromXMLParsers::labelParser.first(input)) {
		label::Label label = alib::api<label::Label>::parse(input);
		tmp = std::move(label.getData()).plunder();
	} else if(alib::FromXMLParsers::regexpParser.first(input)) {
		regexp::RegExp regexp = alib::api<regexp::RegExp>::parse(input);
		tmp = std::move(regexp.getData()).plunder();
	} else if(alib::FromXMLParsers::stringParser.first(input)) {
		string::String string = alib::api<string::String>::parse(input);
		tmp = std::move(string.getData()).plunder();
	} else if(alib::FromXMLParsers::automatonParser.first(input)) {
		automaton::Automaton automaton = alib::api<automaton::Automaton>::parse(input);
		tmp = std::move(automaton.getData()).plunder();
	} else if(alib::FromXMLParsers::grammarParser.first(input)) {
		grammar::Grammar grammar = alib::api<grammar::Grammar>::parse(input);
		tmp = std::move(grammar.getData()).plunder();
	} else if(alib::FromXMLParsers::containerParser.first(input)) {
		container::Container container = alib::api<container::Container>::parse(input);
		tmp = std::move(container.getData()).plunder();
	} else {
		throw exception::AlibException("Unknown element in xml");
	}
	Object res {std::move(*tmp)};
	delete tmp;
	return res;
}

bool ObjectFromXMLParser::first(std::list<sax::Token>& input) {
	return alib::FromXMLParsers::symbolParser.first(input) || alib::FromXMLParsers::exceptionParser.first(input) || alib::FromXMLParsers::labelParser.first(input) || alib::FromXMLParsers::regexpParser.first(input) || alib::FromXMLParsers::stringParser.first(input) || alib::FromXMLParsers::automatonParser.first(input) || alib::FromXMLParsers::grammarParser.first(input) || alib::FromXMLParsers::containerParser.first(input);
}

} /* namespace alib */
