/*
 * RegExpToStringComposer.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef REX_EXP_TO_STRING_COMPOSER_H_
#define REX_EXP_TO_STRING_COMPOSER_H_

#include <string>
#include "RegExp.h"
#include "unbounded/UnboundedRegExpElements.h"
#include "formal/FormalRegExpElements.h"

namespace regexp {

class RegExpToStringComposer : public VisitableRegExpBase::visitor_type, UnboundedRegExpElement::visitor_type, FormalRegExpElement::visitor_type {
	void Visit(void*, const UnboundedRegExpAlternation& alternation);
	void Visit(void*, const UnboundedRegExpConcatenation& concatenation);
	void Visit(void*, const UnboundedRegExpIteration& iteration);
	void Visit(void*, const UnboundedRegExpSymbol& symbol);
	void Visit(void*, const UnboundedRegExpEpsilon& epsilon);
	void Visit(void*, const UnboundedRegExpEmpty& empty);


	void Visit(void*, const FormalRegExpAlternation& alternation);
	void Visit(void*, const FormalRegExpConcatenation& concatenation);
	void Visit(void*, const FormalRegExpIteration& iteration);
	void Visit(void*, const FormalRegExpSymbol& symbol);
	void Visit(void*, const FormalRegExpEpsilon& epsilon);
	void Visit(void*, const FormalRegExpEmpty& empty);


	void Visit(void*, const UnboundedRegExp& empty);
	void Visit(void*, const FormalRegExp& empty);


	enum class Priority {
		ALTERNATION,
		CONCATENATION,
		FACTOR
	};
public:
	/**
	 * Composes string representation of RegExp.
	 * @param regexp RegExp to print
	 * @returns string representation of regexp
	 */
	std::string compose(const RegExp& regexp);
};

} /* namespace regexp */

#endif /* REX_EXP_TO_STRING_COMPOSER_H_ */
