/*
 * FormalRegExpConcatenation.h
 *
 *  Created on: Nov 27, 2013
 *      Author: Martin Zak
 */

#ifndef FORMAL_REG_EXP_CONCATENATION_H_
#define FORMAL_REG_EXP_CONCATENATION_H_

#include <vector>
#include "FormalRegExpElement.h"

namespace regexp {

/**
 * Represents concatenation operator in the regular expression. Contains list of FormalRegExpElement
 * as operands of the operator.
 */
class FormalRegExpConcatenation: public std::acceptor<FormalRegExpConcatenation, FormalRegExpElement, FormalRegExpElement> {
protected:
	/**
	 * @copydoc FormalRegExpElement::clone() const
	 */
	virtual UnboundedRegExpElement* cloneAsUnbounded() const;
	
	FormalRegExpElement* left;
	FormalRegExpElement* right;
	
	/**
	 * @copydoc FormalRegExpElement::testSymbol() const
	 */
	virtual bool testSymbol( const alphabet::Symbol & symbol ) const;
	
	/**
	 * @copydoc FormalRegExpElement::attachRegExp()
	 */
	virtual bool attachRegExp ( const FormalRegExp * regexp );
	
	/**
	 * @copydoc FormalRegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const;

public:
	explicit FormalRegExpConcatenation(FormalRegExpElement&& left, FormalRegExpElement&& right);
	explicit FormalRegExpConcatenation(const FormalRegExpElement& left, const FormalRegExpElement& right);
	
	FormalRegExpConcatenation(const FormalRegExpConcatenation& other);
	FormalRegExpConcatenation(FormalRegExpConcatenation&& other) noexcept;
	FormalRegExpConcatenation& operator =(const FormalRegExpConcatenation& other);
	FormalRegExpConcatenation& operator =(FormalRegExpConcatenation&& other);
	virtual ~FormalRegExpConcatenation() noexcept;
	
	/**
	 * @copydoc FormalRegExpElement::clone() const
	 */
	virtual FormalRegExpElement* clone() const;
	
	/**
	 * @copydoc FormalRegExpElement::plunder() const
	 */
	virtual FormalRegExpElement* plunder() &&;

	/**
	 * @return elements
	 */
	const FormalRegExpElement & getLeftElement() const;
	const FormalRegExpElement & getRightElement() const;

	/**
	 * @return elements
	 */
	FormalRegExpElement& getLeftElement();
	FormalRegExpElement& getRightElement();

	/**
	 * @param element to append
	 */
	void setLeftElement(const FormalRegExpElement& element);
	void setLeftElement(FormalRegExpElement&& element);
	
	void setRightElement(const FormalRegExpElement& element);
	void setRightElement(FormalRegExpElement&& element);

	virtual bool operator<(const FormalRegExpElement&) const;
	virtual bool operator==(const FormalRegExpElement&) const;
	virtual bool operator>(const FormalRegExpElement&) const;

	virtual bool operator<(const FormalRegExpConcatenation&) const;
	virtual bool operator==(const FormalRegExpConcatenation&) const;

	/**
	 * @copydoc FormalRegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;
	
	/**
	 * @copydoc FormalRegExpElement::containsEmptyString() const
	 */
	virtual bool containsEmptyString() const;

	/**
	 * @copydoc FormalRegExpElement::isEmpty() const
	 */
	virtual bool isEmpty() const;
};

} /* namespace regexp */

#endif /* FORMAL_REG_EXP_CONCATENATION_H_ */
