/*
 * UnboundedRegExpSymbol.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "UnboundedRegExpSymbol.h"
#include "../formal/FormalRegExpSymbol.h"

namespace regexp {

UnboundedRegExpSymbol::UnboundedRegExpSymbol(const UnboundedRegExpSymbol& other) : symbol(other.symbol) {

}

UnboundedRegExpSymbol::UnboundedRegExpSymbol(UnboundedRegExpSymbol&& other) noexcept : symbol(std::move(other.symbol)) {
	this->attachRegExp(NULL);
}

UnboundedRegExpSymbol& UnboundedRegExpSymbol::operator=(const UnboundedRegExpSymbol& other) {
	if (this == &other) {
		return *this;
	}

	*this = UnboundedRegExpSymbol(other);

	return *this;
}

UnboundedRegExpSymbol& UnboundedRegExpSymbol::operator=(UnboundedRegExpSymbol&& other) {
	std::swap(this->symbol, other.symbol);
	std::swap(this->parentRegExp, other.parentRegExp);
	
	this->attachRegExp(other.parentRegExp);
	
	return *this;
}

UnboundedRegExpSymbol::UnboundedRegExpSymbol(const alphabet::Symbol& symbol) :
		symbol(symbol) {
}

UnboundedRegExpSymbol::UnboundedRegExpSymbol(alphabet::Symbol&& symbol) :
		symbol(std::move(symbol)) {
}

UnboundedRegExpElement* UnboundedRegExpSymbol::clone() const {
	return new UnboundedRegExpSymbol(*this);
}

UnboundedRegExpElement* UnboundedRegExpSymbol::plunder() && {
	return new UnboundedRegExpSymbol(std::move(*this));
}

FormalRegExpElement* UnboundedRegExpSymbol::cloneAsFormal() const {
	return new FormalRegExpSymbol(this->symbol);
}

bool UnboundedRegExpSymbol::operator==(const alphabet::Symbol& other) const {
	return this->symbol == other;
}

bool operator==(const alphabet::Symbol& first, const UnboundedRegExpSymbol& second) {
	return first == second.symbol;
}

bool UnboundedRegExpSymbol::operator<(const UnboundedRegExpElement& other) const {
	return other > *this;
}

bool UnboundedRegExpSymbol::operator==(const UnboundedRegExpElement& other) const {
	return other == *this;
}

bool UnboundedRegExpSymbol::operator>(const UnboundedRegExpElement& other) const {
	return other < *this;
}


bool UnboundedRegExpSymbol::operator<(const UnboundedRegExpSymbol& other) const {
	return symbol < other.symbol;
}

bool UnboundedRegExpSymbol::operator==(const UnboundedRegExpSymbol& other) const {
	return symbol == other.symbol;
}

void UnboundedRegExpSymbol::operator>>(std::ostream& out) const {
	out << "(UnboundedRegExpSymbol " << symbol << ")";
}

bool UnboundedRegExpSymbol::containsEmptyString() const {
	return false;
}

bool UnboundedRegExpSymbol::isEmpty() const {
	return false;
}

bool UnboundedRegExpSymbol::testSymbol( const alphabet::Symbol & symbol ) const {
	return symbol == this->symbol;
}

bool UnboundedRegExpSymbol::attachRegExp(const UnboundedRegExp * regexp ) {
	if(this->parentRegExp == regexp) return true;
	this->parentRegExp = regexp;
	if(regexp == NULL) return true;
	return this->parentRegExp->getAlphabet().find(this->symbol) != this->parentRegExp->getAlphabet().end();
}

void UnboundedRegExpSymbol::computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const {
	alphabet.insert(this->symbol);
}

const alphabet::Symbol& UnboundedRegExpSymbol::getSymbol() const {
	return this->symbol;
}

} /* namespace regexp */

