/*
 * Epsilon.cpp
 *
 *  Created on: Jan 30, 2014
 *      Author: Jan Travnicek
 */

#include "Epsilon.h"
#include <sstream>
#include <typeinfo>

#include "LinearString.h"
#include "CyclicString.h"

namespace string {

StringBase* Epsilon::clone() const {
	return new Epsilon(*this);
}

StringBase* Epsilon::plunder() && {
	return new Epsilon(std::move(*this));
}

bool Epsilon::removeSymbolFromAlphabet(const alphabet::Symbol & symbol) {
	return alphabet.erase(symbol);
}

bool Epsilon::operator<(const ObjectBase& other) const {
	return other > *this;
}

bool Epsilon::operator==(const ObjectBase& other) const {
	return other == *this;
}

bool Epsilon::operator>(const ObjectBase& other) const {
	return other < *this;
}

std::vector<alphabet::Symbol> Epsilon::getContent() const {
	return {};
}

bool Epsilon::isEmpty() const {
	return true;
}

bool Epsilon::operator<(const Epsilon& other) const {
	return alphabet < other.alphabet;
}

bool Epsilon::operator<(const LinearString& other) const {
	if(other.isEmpty()) return alphabet < other.getAlphabet();
	return typeid(*this).before(typeid(other));
}

bool Epsilon::operator<(const CyclicString& other) const {
	if(other.isEmpty()) return alphabet < other.getAlphabet();
	return typeid(*this).before(typeid(other));
}

bool Epsilon::operator==(const Epsilon& other) const {
	return alphabet == other.alphabet;
}

bool Epsilon::operator==(const LinearString& other) const {
	if(other.isEmpty()) return alphabet == other.getAlphabet();
	return false;
}

bool Epsilon::operator==(const CyclicString& other) const {
	if(other.isEmpty()) return alphabet == other.getAlphabet();
	return false;
}

void Epsilon::operator>>(std::ostream& out) const {
	out << "(Epsilon)";
}

Epsilon::operator std::string() const {
	return "E";
}

Epsilon Epsilon::EPSILON = Epsilon();

} /* namespace string */
