/*
 * LinearString.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef LINEAR_STRING_H_
#define LINEAR_STRING_H_

#include <iostream>
#include <set>
#include <vector>

#include "../std/visitor.hpp"
#include "../alphabet/Symbol.h"
#include "StringBase.h"
#include "common/StringAlphabet.h"

namespace string {

/**
 * Represents regular expression parsed from the XML. Regular expression is stored
 * as a tree of LinearStringElement.
 */
class LinearString : public std::acceptor<LinearString, VisitableStringBase, std::acceptor<LinearString, alib::VisitableObjectBase, StringBase> >, public StringAlphabet {
	std::vector<alphabet::Symbol> m_Data;

public:
	LinearString();
	explicit LinearString(const std::set<alphabet::Symbol>& alphabet, const std::vector<alphabet::Symbol>& data);
	explicit LinearString(std::set<alphabet::Symbol>&& alphabet, std::vector<alphabet::Symbol>&& data);
	explicit LinearString(const std::vector<alphabet::Symbol>& data);
	explicit LinearString(std::vector<alphabet::Symbol>&& data);

	virtual StringBase* clone() const;
	virtual StringBase* plunder() &&;

	virtual bool removeSymbolFromAlphabet(const alphabet::Symbol & symbol);

	/**
	 * @param element to append
	 */
	void appendSymbol(const alphabet::Symbol& symbol);
	
	void appendSymbol(alphabet::Symbol&& symbol);

	virtual bool operator <(const alib::ObjectBase& other) const;
	virtual bool operator ==(const alib::ObjectBase& other) const;
	virtual bool operator >(const alib::ObjectBase& other) const;

	/**
	 * @return List of symbols forming string (const version).
	 */
	const std::vector<alphabet::Symbol>& getContent() const;

	void setContent(std::vector<alphabet::Symbol> data);

	/**
	 * @return true if string is an empty word (vector length is 0)
	 */
	bool isEmpty() const;

	virtual bool operator<(const LinearString& other) const;
	virtual bool operator<(const CyclicString& other) const;
	virtual bool operator<(const Epsilon& other) const;

	virtual bool operator==(const LinearString& other) const;
	virtual bool operator==(const CyclicString& other) const;
	virtual bool operator==(const Epsilon& other) const;

	virtual void operator >>(std::ostream& out) const;

	virtual operator std::string() const;

	virtual int selfTypeId() const {
		return typeId(*this);
	}
};

} /* namespace string */

#endif /* LINEAR_STRING_H_ */

