/*
 * StringFromXMLParser.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef STRING_FROM_XML_PARSER_H_
#define STRING_FROM_XML_PARSER_H_

#include "../sax/FromXMLParserHelper.h"
#include <vector>
#include <set>
#include "String.h"
#include "StringFeatures.h"
#include "Epsilon.h"
#include "LinearString.h"
#include "CyclicString.h"
#include "../sax/Token.h"
#include "../alphabet/Symbol.h"

namespace alib {

template<typename T>
struct api;

} /* namespace alib */

namespace string {

/**
 * Parser used to get String from XML parsed into list of tokens.
 */
class StringFromXMLParser : public sax::FromXMLParserHelper {
	std::vector<alphabet::Symbol> parseContentData(std::list<sax::Token>& input) const;
	std::set<alphabet::Symbol> parseAlphabet(std::list<sax::Token> &input) const;

	String parseString(std::list<sax::Token>& input) const;
	String parseString(std::list<sax::Token>& input, const std::set<FEATURES>& features) const;

	LinearString parseLinearString(std::list<sax::Token>& input) const;
	CyclicString parseCyclicString(std::list<sax::Token>& input) const;
	Epsilon parseEpsilon(std::list<sax::Token>& input) const;

	template<typename T> friend class alib::api;
public:
	bool first(std::list<sax::Token>& input) const;
};

} /* namespace string */

#endif /* STRING_FROM_XML_PARSER_H_ */
