#include <list>
#include "SymbolTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "alphabet/Symbol.h"
#include "alphabet/LabeledSymbol.h"
#include "alphabet/BlankSymbol.h"
#include "alphabet/SymbolFromStringParser.h"
#include "alphabet/SymbolToStringComposer.h"
#include "alphabet/SymbolFromXMLParser.h"
#include "alphabet/SymbolToXMLComposer.h"

#include "factory/DataFactory.hpp"

#include "label/StringLabel.h"
#include "label/Label.h"

#define CPPUNIT_IMPLY(x, y) CPPUNIT_ASSERT(!(x) || (y))
#define CPPUNIT_EXCLUSIVE_OR(x, y) CPPUNIT_ASSERT((!(x) && (y)) || ((x) && !(y)))

CPPUNIT_TEST_SUITE_REGISTRATION( SymbolTest );

void SymbolTest::setUp() {
}

void SymbolTest::tearDown() {
}

void SymbolTest::testCopyConstruct() {
  alphabet::Symbol symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("1"))));
  
  alphabet::Symbol symbol2(symbol);
  
  CPPUNIT_ASSERT( symbol == symbol2 );
  
  alphabet::Symbol symbol3(std::move(symbol));
  
  CPPUNIT_ASSERT( symbol2 == symbol3 );
}

void SymbolTest::testEqual() {
  {
    std::string input = "a";
    std::stringstream inputs(input);

    alphabet::SymbolFromStringParser parser(inputs);
    alphabet::Symbol symbol = parser.parseValue();

    alphabet::SymbolToStringComposer composer;
    std::string output = composer.compose(symbol);
    std::stringstream outputs(output);

    CPPUNIT_ASSERT( input == output );

    alphabet::SymbolFromStringParser parser2(outputs);
    alphabet::Symbol symbol2 = parser2.parseValue();

    CPPUNIT_ASSERT( symbol == symbol2 );
  }
  {
    std::string input = "'aaaa'";
    std::stringstream inputs(input);

    alphabet::SymbolFromStringParser parser(inputs);
    alphabet::Symbol symbol = parser.parseValue();

    alphabet::SymbolToStringComposer composer;
    std::string output = composer.compose(symbol);
    std::stringstream outputs(output);

    CPPUNIT_ASSERT( input == output );

    alphabet::SymbolFromStringParser parser2(outputs);
    alphabet::Symbol symbol2 = parser2.parseValue();

    CPPUNIT_ASSERT( symbol == symbol2 );
  }
  {
    std::string input = "#B";
    std::stringstream inputs(input);

    alphabet::SymbolFromStringParser parser(inputs);
    alphabet::Symbol symbol = parser.parseValue();

    alphabet::SymbolToStringComposer composer;
    std::string output = composer.compose(symbol);
    std::stringstream outputs(output);

    CPPUNIT_ASSERT( input == output );

    alphabet::SymbolFromStringParser parser2(outputs);
    alphabet::Symbol symbol2 = parser2.parseValue();

    CPPUNIT_ASSERT( symbol == symbol2 );
  }
}

void SymbolTest::testXMLParser() {

  alphabet::Symbol symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("1"))));

  {
	std::list<sax::Token> tokens = alib::DataFactory::toTokens(symbol);
	std::string tmp;
	sax::SaxComposeInterface::printMemory(tmp, tokens);
  
	std::list<sax::Token> tokens2;
	sax::SaxParseInterface::parseMemory(tmp, tokens2);
	alphabet::Symbol symbol2 = alib::DataFactory::fromTokens<alphabet::Symbol>(tokens2);
  
	CPPUNIT_ASSERT( symbol == symbol2 );
  }
  {
	std::string tmp = alib::DataFactory::toString(symbol);
	alphabet::Symbol symbol2 = alib::DataFactory::fromString<alphabet::Symbol>(tmp);
  
	CPPUNIT_ASSERT( symbol == symbol2 );
  }
}

void SymbolTest::testOrder() {
  alphabet::LabeledSymbol ls(alphabet::LabeledSymbol(label::Label(label::StringLabel("a"))));
  alphabet::BlankSymbol bs;
  
  CPPUNIT_EXCLUSIVE_OR( bs < ls, ls < bs);
}

