#include <list>
#include "LabelTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "label/Label.h"
#include "label/StringLabel.h"
#include "label/IntegerLabel.h"
#include "label/CharacterLabel.h"
#include "label/LabelFromStringParser.h"
#include "label/LabelToStringComposer.h"
#include "label/LabelFromXMLParser.h"
#include "label/LabelToXMLComposer.h"

#include "factory/DataFactory.hpp"

#define CPPUNIT_EXCLUSIVE_OR(x, y) CPPUNIT_ASSERT((!(x) && (y)) || ((x) && !(y)))

CPPUNIT_TEST_SUITE_REGISTRATION( LabelTest );

void LabelTest::setUp() {
}

void LabelTest::tearDown() {
}

void LabelTest::testCopyConstruct() {
  label::Label label(label::StringLabel("1"));
  
  label::Label label2(label);
  
  CPPUNIT_ASSERT( label == label2 );
  
  label::Label label3(std::move(label));
  
  CPPUNIT_ASSERT( label2 == label3 );
}

void LabelTest::testEqual() {
  {
    std::string input = "1";
    std::stringstream inputs(input);

    label::LabelFromStringParser parser(inputs);
    label::Label label = parser.parseValue();

    label::LabelToStringComposer composer;
    std::string output = composer.compose(label);
    std::stringstream outputs(output);

    CPPUNIT_ASSERT( input == output );

    label::LabelFromStringParser parser2(outputs);
    label::Label label2 = parser2.parseValue();

    CPPUNIT_ASSERT( label == label2 );
  }
  {
    std::string input = "'aaaa'";
    std::stringstream inputs(input);

    label::LabelFromStringParser parser(inputs);
    label::Label label = parser.parseValue();

    label::LabelToStringComposer composer;
    std::string output = composer.compose(label);
    std::stringstream outputs(output);

    CPPUNIT_ASSERT( input == output );

    label::LabelFromStringParser parser2(outputs);
    label::Label label2 = parser2.parseValue();

    CPPUNIT_ASSERT( label == label2 );
  }
  {
    std::string input = "[1, 2, 3]";
    std::stringstream inputs(input);

    label::LabelFromStringParser parser(inputs);
    label::Label label = parser.parseValue();

    label::LabelToStringComposer composer;
    std::string output = composer.compose(label);
    std::stringstream outputs(output);

    CPPUNIT_ASSERT( input == output );

    label::LabelFromStringParser parser2(outputs);
    label::Label label2 = parser2.parseValue();

    CPPUNIT_ASSERT( label == label2 );
  }
  {
    std::string input = "<1, 2>";
    std::stringstream inputs(input);

    label::LabelFromStringParser parser(inputs);
    label::Label label = parser.parseValue();

    label::LabelToStringComposer composer;
    std::string output = composer.compose(label);
    std::stringstream outputs(output);

    CPPUNIT_ASSERT( input == output );

    label::LabelFromStringParser parser2(outputs);
    label::Label label2 = parser2.parseValue();

    CPPUNIT_ASSERT( label == label2 );
  }
}

void LabelTest::testXMLParser() {

  label::Label label(label::StringLabel("1"));

  {
	std::list<sax::Token> tokens = alib::DataFactory::toTokens(label);
	std::string tmp;
	sax::SaxComposeInterface::printMemory(tmp, tokens);
  
	std::list<sax::Token> tokens2;
	sax::SaxParseInterface::parseMemory(tmp, tokens2);
	label::Label label2 = alib::DataFactory::fromTokens<label::Label>(tokens2);
  
	CPPUNIT_ASSERT( label == label2 );
  }
  {
	std::string tmp = alib::DataFactory::toString(label);
	label::Label label2 = alib::DataFactory::fromString<label::Label>(tmp);
  
	CPPUNIT_ASSERT( label == label2 );
  }
}

void LabelTest::testOrder() {
  label::StringLabel s1("1");
  label::CharacterLabel c1('1');
  label::IntegerLabel i1(1);
  
  CPPUNIT_EXCLUSIVE_OR( s1 < i1, i1 < s1);
  CPPUNIT_EXCLUSIVE_OR( c1 < i1, i1 < c1);
  CPPUNIT_EXCLUSIVE_OR( s1 < c1, c1 < s1);
}

