//============================================================================
// Name        : aminimize.cpp
// Author      : Jan Travnicek
//============================================================================

#include <iostream>
#include <string>
#include <set>

#include "exception/AlibException.h"
#include "factory/DataFactory.hpp"
#include "minimize/dfa/MinimizeDFA.h"

int main(int argc, char** argv) {

	try {

		automaton::DFA* automatonPointer = NULL;

		if (argc == 2 && std::string("-h").compare(argv[1]) == 0) {
			std::cout << "Automaton minimize." << std::endl << "Usage: aminimize automaton.xml" << std::endl;
			return -1;
		} else if (argc == 1 || (argc == 2 && std::string("--").compare(argv[1]) == 0)) {
			automatonPointer = static_cast<automaton::DFA*>(std::move(alib::DataFactory::fromStdin<automaton::DFA>()).plunder());
		} else if (argc == 2) {
			automatonPointer = static_cast<automaton::DFA*>(std::move(alib::DataFactory::fromFile<automaton::DFA>(argv[1])).plunder());
		} else {
			std::cout << "Automaton minimize require deterministic finite automaton" << std::endl;
			return 1;
		}
		
		automaton::DFA res = minimize::MinimizeDFA::minimize(*automatonPointer);
		alib::DataFactory::toStdout(res);

		delete automatonPointer;

		return 0;

	} catch (const exception::AlibException& exception) {
		alib::DataFactory::toStdout(exception);
		return 1;
	} catch(...) {
		return 127;
	}
}
