/*
 * arand.cpp
 *
 *  Created on: 26. 3. 2014
 *	  Author: tomas
 */

#include <iostream>
#include <cstdlib>
#include <climits>
#include <cfloat>
#include <getopt.h>

#include <automaton/FSM/NFA.h>
#include <factory/DataFactory.hpp>
#include "generator/RandomAutomatonFactory.h"

void help( void )
{
	std::cout << "arand" << std::endl;
	std::cout << "Generates random automaton." << std::endl;
	std::cout << "Usage: arand -n=N -d=D -a=A" << std::endl << std::endl;
	std::cout << "  -n, --nodes=N \t Number of nodes of automata." << std::endl;
	std::cout << "  -d, --density=D \t Percentage of transitions to generate. 1.0 = automata graph is tree, 100.0 = every possible transition." << std::endl;
	std::cout << "  -a, --alphabet=A \t Alphabet size." << std::endl;
}


int main(int argc, char* argv[])
{
	static struct option long_options[] = {
	  // common
	  {"help",	no_argument,		NULL, 'h'},
//	  {"type",	required_argument,	NULL, 't'},

	  // automaton options
	  {"density",		required_argument,	NULL,	'd' },
	  {"nodes",		required_argument,	NULL,	'n' },
	  {"alphabet",		required_argument,	NULL,	'a' },
	  {0,			0,			0,	'\0'}
	};

	int long_index = 0, opt = 0;

	// string type;
	int statesCount = INT_MAX, alphabetSize = INT_MAX;
	double density = DBL_MAX;

	while( ( opt = getopt_long( argc, argv,"ht:d:n:a:", long_options, & long_index ) )  != -1 )
	{
		switch( opt )
		{
		/*
		case 't':
			type.assign( optarg, strlen( optarg ) );
			break;
		 */

		case 'n':
			statesCount = std::stoi( optarg, nullptr, 10 );
			break;

		case 'd':
			density = std::stod( optarg, nullptr );
			break;

		case 'a':
			alphabetSize = std::stoi( optarg, nullptr, 10 );
			break;

		case 'v':
		case 'h':
		default:
			help( );
			return 0;
		}
	}

	if( density < 1 || density > 100 )
	{
		//TODO: floating point arithmetic
		std::cerr << "You must specify density as a number between 1 and 100." << std::endl;
		return 1;

	}

	if( statesCount == INT_MAX )
	{
		std::cerr << "You must specify number of states." << std::endl;
		return 1;
	}

	if( alphabetSize == INT_MAX )
	{
		std::cerr << "You must specify alphabet or at least its size." << std::endl;
		return 1;
	}

	automaton::NFA res = generator::RandomAutomatonFactory::generateNFA( statesCount, alphabetSize, density );
	alib::DataFactory::toStdout(res);

	return 0;
}
